@@ -29,13 +29,6 @@
  * 4. This notice may not be removed or altered.
  */
 
-#ifndef lint
-static char copyright[] =
-"@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: arg.c,v 1.51 2012/04/10 16:30:06 abe Exp abe $";
-#endif
-
-
 #include "lsof.h"
 
 
@@ -118,7 +111,7 @@ FD_range_nondigit:
  */
 
 int
-ck_file_arg(i, ac, av, fv, rs, sbp)
+ck_file_arg(i, ac, av, fv, rs, sbp, accept_deleted_file)
 	int i;			/* first file argument index */
 	int ac;			/* argument count */
 	char *av[];		/* argument vector */
@@ -127,6 +120,8 @@ ck_file_arg(i, ac, av, fv, rs, sbp)
 				 *	0 = undone; 1 = done */
 	struct stat *sbp;	/* if non-NULL, pointer to stat(2) buffer
 				 * when argument count == 1 */
+	int accept_deleted_file; /* if non-zero, don't report an error even
+				  * when the file doesn't exist. */
 {
 	char *ap, *fnm, *fsnm, *path;
 	short err = 0;
@@ -177,7 +172,7 @@ ck_file_arg(i, ac, av, fv, rs, sbp)
 		    if (!(ap = (char *)malloc((MALLOC_S)(k + 1)))) {
 			(void) fprintf(stderr, "%s: no space for copy of %s\n",
 			    Pn, path);
-			Exit(1);
+			Error();
 		    }
 		    (void) strncpy(ap, path, k);
 		    ap[k] = '\0';
@@ -228,14 +223,16 @@ ck_file_arg(i, ac, av, fv, rs, sbp)
 		    if (!mmp) {
 			(void) fprintf(stderr,
 			    "%s: no space for mount pointers\n", Pn);
-			Exit(1);
+			Error();
 		    }
 		}
 		mmp[nm++] = mp;
 	    }
 	    if (fv == 2 && nm == 0) {
-		(void) fprintf(stderr, "%s: not a file system: ", Pn);
-		safestrprt(av[i], stderr, 1);
+		if (!accept_deleted_file) {
+		    (void) fprintf(stderr, "%s: not a file system: ", Pn);
+		    safestrprt(av[i], stderr, 1);
+		}
 		ErrStat = 1;
 		continue;
 	    }
@@ -251,7 +248,7 @@ ck_file_arg(i, ac, av, fv, rs, sbp)
 	     */
 		if (!(sfp = (struct sfile *)malloc(sizeof(struct sfile)))) {
 		    (void) fprintf(stderr, "%s: no space for files\n", Pn);
-		    Exit(1);
+		    Error();
 		}
 		sfp->next = Sfile;
 		Sfile = sfp;
@@ -272,10 +269,11 @@ ck_file_arg(i, ac, av, fv, rs, sbp)
 		    else {
 			if (statsafely(fnm, &sb) != 0) {
 			    int en = errno;
-
-			    (void) fprintf(stderr, "%s: status error on ", Pn);
-			    safestrprt(fnm, stderr, 0);
-			    (void) fprintf(stderr, ": %s\n", strerror(en));
+			    if (!accept_deleted_file) {
+				(void) fprintf(stderr, "%s: status error on ", Pn);
+				safestrprt(fnm, stderr, 0);
+				(void) fprintf(stderr, ": %s\n", strerror(en));
+			    }
 			    Sfile = sfp->next;
 			    (void) free((FREE_P *)sfp);
 			    ErrStat = 1;
@@ -289,7 +287,7 @@ ck_file_arg(i, ac, av, fv, rs, sbp)
 		    }
 		    sfp->i = (INODETYPE)sb.st_ino;
 		    sfp->mode = sb.st_mode & S_IFMT;
-		
+
 #if	defined(CKFA_EXPDEV)
 		/*
 		 * Expand device numbers before saving, so that they match the
@@ -363,7 +361,7 @@ ck_file_arg(i, ac, av, fv, rs, sbp)
 			(void) fprintf(stderr,
 			    "%s: no space for file name: ", Pn);
 			safestrprt(fnm, stderr, 1);
-			Exit(1);
+			Error();
 		    }
 
 #if	defined(HASPROCFS)
@@ -383,7 +381,7 @@ ck_file_arg(i, ac, av, fv, rs, sbp)
 			(void) fprintf(stderr,
 			    "%s: no space for file system name: ", Pn);
 			safestrprt(fsnm, stderr, 1);
-			Exit(1);
+			Error();
 		    }
 
 #if	defined(HASPROCFS)
@@ -395,7 +393,7 @@ ck_file_arg(i, ac, av, fv, rs, sbp)
 		    (void) fprintf(stderr,
 			"%s: no space for argument file name: ", Pn);
 			safestrprt(av[i], stderr, 1);
-		    Exit(1);
+		    Error();
 		}
 
 #if	defined(HASPROCFS)
@@ -447,7 +445,7 @@ ck_file_arg(i, ac, av, fv, rs, sbp)
 		    (void) fprintf(stderr, "%s: no space for %s ID: ",
 			Pn, Mtprocfs->dir);
 		    safestrprt(path, stderr, 1);
-		    Exit(1);
+		    Error();
 		}
 		pfi->pid = pid;
 		pfi->f = 0;
@@ -472,8 +470,15 @@ ck_file_arg(i, ac, av, fv, rs, sbp)
 
 	    } while (mx < nm);
 	}
-	if (!ss)
+
+	if (accept_deleted_file) {
+	    if (!ss && ErrStat == 0)
+		err = 1;
+	    if (ErrStat)
+		ErrStat = 0;
+	} else if (!ss) {
 	    err = 1;
+	}
 	return((int)err);
 }
 
@@ -488,7 +493,7 @@ ctrl_dcache(c)
 	char *c;			/* control string */
 {
 	int rc = 0;
-	
+
 	if (!c) {
 	    (void) fprintf(stderr,
 		"%s: no device cache option control string\n", Pn);
@@ -564,7 +569,7 @@ ctrl_dcache(c)
 	    if (!(DCpathArg = mkstrcpy(c, (MALLOC_S *)NULL))) {
 		(void) fprintf(stderr, "%s: no space for -D path: ", Pn);
 		safestrprt(c, stderr, 1);
-		Exit(1);
+		Error();
 	    }
 	}
 	return(0);
@@ -678,7 +683,7 @@ enter_cmd_rx(x)
 	if (!(xp = (char *)malloc(xl + 1))) {
 	    (void) fprintf(stderr, "%s: no regexp space for: ", Pn);
 	    safestrprt(x, stderr, 1);
-	    Exit(1);
+	    Error();
 	}
 	(void) strncpy(xp, xb, xl);
 	xp[(int)xl] = '\0';
@@ -699,7 +704,7 @@ enter_cmd_rx(x)
 	    if (!CmdRx) {
 		(void) fprintf(stderr, "%s: no space for regexp: ", Pn);
 		safestrprt(x, stderr, 1);
-		Exit(1);
+		Error();
 	    }
 	}
 	i = NCmdRxU;
@@ -754,7 +759,7 @@ enter_efsys(e, rdlnk)
 	if (!(ec = mkstrcpy(e, (MALLOC_S *)NULL))) {
 	    (void) fprintf(stderr, "%s: no space for -e string: ", Pn);
 	    safestrprt(e, stderr, 1);
-	    Exit(1);
+	    Error();
 	}
 	if (rdlnk)
 	    path = ec;
@@ -772,13 +777,15 @@ enter_efsys(e, rdlnk)
  * Enter file system path on list, avoiding duplicates.
  */
 	for (ep = Efsysl; ep; ep = ep->next) {
-	   if (!strcmp(ep->path, path))
-		return(0);
+	   if (!strcmp(ep->path, path)) {
+		(void)free((FREE_P *)path);
+		return (0);
+	   }
 	}
 	if (!(ep = (efsys_list_t *)malloc((MALLOC_S)(sizeof(efsys_list_t))))) {
 	   (void) fprintf(stderr, "%s: no space for \"-e %s\" entry\n",
 		Pn, e);
-	   Exit(1);
+	   Error();
 	}
 	ep->path = path;
 	ep->pathl = i;
@@ -812,7 +819,7 @@ enter_fd(f)
 	if (!(fc = mkstrcpy(f, (MALLOC_S *)NULL))) {
 	    (void) fprintf(stderr, "%s: no space for fd string: ", Pn);
 	    safestrprt(f, stderr, 1);
-	    Exit(1);
+	    Error();
 	}
 /*
  * Isolate each file descriptor in the comma-separated list, then enter it
@@ -909,7 +916,7 @@ enter_fd_lst(nm, lo, hi, excl)
  */
 	if (!(f = (struct fd_lst *)malloc((MALLOC_S)sizeof(struct fd_lst)))) {
 	   (void) fprintf(stderr, "%s: no space for FD list entry\n", Pn);
-	   Exit(1);
+	   Error();
 	}
 	if (nm) {
 
@@ -930,7 +937,7 @@ enter_fd_lst(nm, lo, hi, excl)
 		if (!(f->nm = mkstrcpy(nm, (MALLOC_S *)NULL))) {
 		    (void) fprintf(stderr,
 			"%s: no space for copy of: %s\n", Pn, nm);
-		    Exit(1);
+		    Error();
 		}
 		lo = 1;
 		hi = 0;
@@ -957,6 +964,12 @@ enter_fd_lst(nm, lo, hi, excl)
  */
 	f->hi = hi;
 	f->lo = lo;
+	if (f->nm && strcmp(f->nm, "fd") == 0) {
+	    (void) free((FREE_P *)f->nm);
+	    f->nm = NULL;
+	    f->hi = INT_MAX;
+	    f->lo = 0;
+	}
 	f->next = Fdl;
 	Fdl = f;
 	FdlTy = excl;
@@ -1040,7 +1053,7 @@ enter_dir(d, descend)
 	av[0] = (dn == d) ? mkstrcpy(dn, (MALLOC_S *)NULL) : dn;
 	av[1] = (char *)NULL;
 	dn = (char *)NULL;
-	if (!ck_file_arg(0, 1, av, 1, 1, &sb)) {
+	if (!ck_file_arg(0, 1, av, 1, 1, &sb, 0)) {
 	    av[0] = (char *)NULL;
 	    fct++;
 	}
@@ -1084,7 +1097,7 @@ enter_dir(d, descend)
 		    (void) fprintf(stderr,
 			"%s: no space for path to entries in directory: %s\n",
 			Pn, dn);
-		    Exit(1);
+		    Error();
 		}
 	    }
 	    (void) snpf(fp, (size_t)fpl, "%s%s", dn, sl ? "/" : "");
@@ -1129,7 +1142,7 @@ enter_dir(d, descend)
 			safestrprt(dn, stderr, 0);
 			putc('/', stderr);
 			safestrprtn(dp->d_name, dnamlen, stderr, 1);
-			Exit(1);
+			Error();
 		    }
 		}
 		(void) strncpy(fp + dnl + sl, dp->d_name, dnamlen);
@@ -1203,7 +1216,7 @@ enter_dir(d, descend)
 	     * Use ck_file_arg() to record the entry for searching.  Force it
 	     * to consider the entry a file, not a file system.
 	     */
-		if (!ck_file_arg(0, 1, av, 1, 1, &sb)) {
+		if (!ck_file_arg(0, 1, av, 1, 1, &sb, 0)) {
 		    av[0] = (char *)NULL;
 		    fct++;
 		}
@@ -1289,7 +1302,7 @@ enter_id(ty, p)
 	    (void) fprintf(stderr, "%s: enter_id \"", Pn);
 	    safestrprt(p, stderr, 0);
 	    (void) fprintf(stderr, "\", invalid type: %d\n", ty);
-	    Exit(1);
+	    Error();
 	}
 /*
  * Convert and store the ID.
@@ -1358,7 +1371,7 @@ enter_id(ty, p)
 		if (!s) {
 		    (void) fprintf(stderr, "%s: no space for %d process%s IDs",
 			Pn, mx, (ty == PGID) ? " group" : "");
-		    Exit(1);
+		    Error();
 		}
 	    }
 	    s[n].f = 0;
@@ -1639,7 +1652,7 @@ unacc_address:
 			n.af = AF_INET;
 		    he = lkup_hostnm(hn, &n);
 #endif	/* defined(HASIPv6) */
-		
+
 		    if (!he) {
 			fprintf(stderr, "%s: unknown host name (%s) in: -i ",
 			    Pn, hn);
@@ -1726,7 +1739,7 @@ unacc_port:
 		     * If no protocol has been specified, look up the port
 		     * numbers for the service name for both TCP and UDP.
 		     */
-			if((se = getservbyname(sn, "tcp")))
+			if ((se = getservbyname(sn, "tcp")))
 			    pt = (int)ntohs(se->s_port);
 			if ((se1 = getservbyname(sn, "udp")))
 			    pu = (int)ntohs(se1->s_port);
@@ -1847,7 +1860,7 @@ enter_nwad(n, sp, ep, s, he)
 		(void) fprintf(stderr,
 		    "%s: no space for Internet argument: -i ", Pn);
 		safestrprt(s, stderr, 1);
-		Exit(1);
+		Error();
 	    }
 	} else
 	    n->arg = (char *)NULL;
@@ -2002,7 +2015,7 @@ no_IorX_space:
 
 			(void) fprintf(stderr, "%s: no %s table space\n",
 			    Pn, ty);
-			Exit(1);
+			Error();
 		    }
 		}
 		if (!UdpStX) {
@@ -2043,12 +2056,12 @@ no_IorX_space:
 	if (!(ssc = mkstrcpy(cp, (MALLOC_S *)NULL))) {
 	    (void) fprintf(stderr,
 		"%s: no temporary state argument space for: %s\n", Pn, ss);
-	    Exit(1);
+	    Error();
 	}
 	cp = ssc;
 	err = 0;
 	while (*cp) {
-	
+
 	/*
 	 * Determine inclusion or exclusion for this state name.
 	 */
@@ -2337,14 +2350,14 @@ enter_uid(us)
 		    Suid = (struct seluid *)realloc((MALLOC_P *)Suid, len);
 		if (!Suid) {
 		    (void) fprintf(stderr, "%s: no space for UIDs", Pn);
-		    Exit(1);
+		    Error();
 		}
 	    }
 	    if (nn) {
 		if (!(lp = mkstrcpy(lnm, (MALLOC_S *)NULL))) {
 		    (void) fprintf(stderr, "%s: no space for login: ", Pn);
 		    safestrprt(lnm, stderr, 1);
-		    Exit(1);
+		    Error();
 		}
 		Suid[Nuid].lnm = lp;
 	    } else
