@@ -32,7 +32,6 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dproc.c,v 1.26 2008/10/21 16:14:18 abe Exp $";
 #endif
 
 
@@ -252,7 +251,7 @@ ckkv(d, er, ev, ea)
 
 #define	OSLEVEL		"oslevel"
 #define	OSLEVELPATH	"/usr/bin/oslevel"
-	
+
 	if (stat(OSLEVELPATH, &sb)) {
 
 #  if	AIXV<4100
@@ -262,12 +261,12 @@ ckkv(d, er, ev, ea)
 
 	    (void) fprintf(stderr, "%s: can't execute %s: %s\n",
 		Pn, OSLEVELPATH, strerror(errno));
-	    Exit(1);
+	    Error();
 	}
 	if ((sb.st_mode & (S_IROTH | S_IXOTH)) != (S_IROTH | S_IXOTH)) {
 	    (void) fprintf(stderr, "%s: can't execute %s, modes: %o\n",
 		Pn, OSLEVELPATH, sb.st_mode);
-	    Exit(1);
+	    Error();
 	}
 /*
  * Open a pipe for receiving the version number from OSLEVEL.  Fork a
@@ -276,7 +275,7 @@ ckkv(d, er, ev, ea)
 	if (pipe(p)) {
 	    (void) fprintf(stderr, "%s: can't create pipe to: %s\n",
 		Pn, OSLEVELPATH);
-	    Exit(1);
+	    Error();
 	}
 	if ((pid = fork()) == 0) {
 	    (void) close(1);
@@ -291,7 +290,7 @@ ckkv(d, er, ev, ea)
 	if (pid < 0) {
 	    (void) fprintf(stderr, "%s: can't fork a child for %s: %s\n",
 		Pn, OSLEVELPATH, strerror(errno));
-	    Exit(1);
+	    Error();
 	}
 	(void) close(p[1]);
 	br = read(p[0], buf, sizeof(buf) - 1);
@@ -409,7 +408,7 @@ gather_proc_info()
 	     * If only ORed process selection options have been specified,
 	     * enable conditional file skipping and socket file only checking.
 	     */
-		if ((Selflags & SELFILE) || !(Selflags & SELPROC))
+		if ((Selflags & SELFILE) || !(Selflags & SelProc))
 		    cckreg = ckscko = 0;
 		else
 		    cckreg = ckscko = 1;
@@ -431,7 +430,7 @@ gather_proc_info()
 	    if (!(P = (struct PROCINFO *)malloc((MALLOC_S)PROCSIZE))) {
 		(void) fprintf(stderr,
 		    "%s: can't allocate space for 1 proc\n", Pn);
-		Exit(1);
+		Error();
 	    }
 	    Np = 1;
 	}
@@ -442,7 +441,7 @@ gather_proc_info()
 	    {
 		(void) fprintf(stderr, "%s: no space for %d procinfo's\n",
 		    Pn, Np);
-		Exit(1);
+		Error();
 	    }
 	}
 #else	/* AIXV>=4300 */
@@ -452,7 +451,7 @@ gather_proc_info()
 		(void) fprintf(stderr,
 		    "%s: can't allocate space for %d procs\n",
 		    Pn, PROCINFO_INCR);
-		Exit(1);
+		Error();
 	    }
 	    Np = PROCINFO_INCR;
 	}
@@ -467,7 +466,7 @@ gather_proc_info()
 		if (!(P = (struct PROCINFO *)realloc((MALLOC_P *)P, msz))) {
 		    (void) fprintf(stderr,
 			"%s: no more space for proc storage\n", Pn);
-		    Exit(1);
+		    Error();
 		}
 		Np += PROCINFO_INCR;
 	    }
@@ -537,9 +536,9 @@ gather_proc_info()
 	     * socket file only checking, based on the process' selection
 	     * status.
 	     */
-		ckscko = (sf & SELPROC) ? 0 : 1;
+		ckscko = (sf & SelProc) ? 0 : 1;
 	    }
-	    
+
 #else	/* AIXV>=4300 */
 	/*
 	 * For AIX 4.3 and above, skip processes excluded by the procsinfo
@@ -557,7 +556,7 @@ gather_proc_info()
 	     * socket file only checking, based on the process' selection
 	     * status.
 	     */
-		ckscko = (sf & SELPROC) ? 0 : 1;
+		ckscko = (sf & SelProc) ? 0 : 1;
 	    }
 	    if (!fds) {
 		if (!(fds = (struct FDSINFO *)malloc((MALLOC_S)FDSINFOSIZE)))
@@ -565,7 +564,7 @@ gather_proc_info()
 		    (void) fprintf(stderr,
 			"%s: can't allocate fdsinfo struct for PID %d\n",
 			Pn, pid);
-		    Exit(1);
+		    Error();
 		}
 	    }
 	    pid = p->p_pid;
@@ -603,7 +602,7 @@ gather_proc_info()
 		 * from the PROCINFO structure.  Try the offsets 0, 48, and
 		 * -48.  Note: these offsets are heuristic attempts to adjust
 		 * to differences in the user struct as observed on two systems
-		 * whose <sys/user.h> header files differed.  U_maxofile 
+		 * whose <sys/user.h> header files differed.  U_maxofile
 		 * follows U_loader by the same number of elements in both
 		 * user structs, so the U_loader offset should be the same as
 		 * the U_maxofile offset.
@@ -804,7 +803,7 @@ get_kernel_access()
 		"%s: FATAL: compiled for a kernel of %s bit size.\n", Pn, kbb);
 	    (void) fprintf(stderr,
 		"      The bit size of this kernel is %s.\n", kbr);
-	    Exit(1);
+	    Error();
 	}
 #endif	/* defined(AIX_KERNBITS) */
 
@@ -829,7 +828,7 @@ get_kernel_access()
  * See if the non-KMEM memory file is readable.
  */
 	if (Memory && !is_readable(Memory, 1))
-	    Exit(1);
+	    Error();
 #endif	/* defined(WILLDROPGID) */
 
 /*
@@ -841,7 +840,7 @@ get_kernel_access()
 	    oe++;
 	}
 	if (oe)
-	    Exit(1);
+	    Error();
 
 #if	defined(WILLDROPGID)
 /*
@@ -858,7 +857,7 @@ get_kernel_access()
 	{
 	    (void) fprintf(stderr, "%s: can't get kernel's %s address\n",
 		Pn, Nl[X_UADDR].n_name);
-	    Exit(1);
+	    Error();
 	}
 
 #if	defined(HAS_AFS)
@@ -1046,7 +1045,7 @@ getsoinfo()
 					    sizeof(so_hash_t *))))
 	{
 	    (void) fprintf(stderr, "%s: no space for *.so hash buckets\n", Pn);
-	    Exit(1);
+	    Error();
 	}
 /*
  * Cache the loader module information, complete with stat(2) results.
@@ -1070,7 +1069,7 @@ getsoinfo()
 	    if (!(rn = mkstrcpy(lp->mi_name, (MALLOC_S *)NULL))) {
 		(void) fprintf(stderr, "%s: no space for name: %s\n", Pn,
 		    lp->mi_name);
-		Exit(1);
+		Error();
 	    }
 	/*
 	 * Resolve symbolic links.
@@ -1093,7 +1092,7 @@ getsoinfo()
 	    if (!(sp = (so_hash_t *)malloc((MALLOC_S)sizeof(so_hash_t)))) {
 		(void) fprintf(stderr, "%s: no space for *.so hash entry: %s\n",
 		    Pn, ln);
-		Exit(1);
+		Error();
 	    }
 	    sp->dev = sb.st_dev;
 	    sp->nlink = (int)sb.st_nlink;
@@ -1202,7 +1201,7 @@ lowpgsp(sig)
 	int sig;
 {
 	(void) fprintf(stderr, "%s: FATAL: system paging space is low.\n", Pn);
-	Exit(1);
+	Error();
 }
 #endif	/* defined(SIGDANGER) */
 
@@ -1312,7 +1311,7 @@ process_text(sid)
 		if (!f) {
 		    (void) fprintf(stderr,
 			"%s: no space for text file pointers\n", Pn);
-		    Exit(1);
+		    Error();
 		}
 	    }
 	    f[n++] = le->fp;
