@@ -36,12 +36,15 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 2005-2007 Apple Inc. and Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dfile.c,v 1.8 2012/04/10 16:41:04 abe Exp abe $";
 #endif
 
 
 #include "lsof.h"
 
+#if	defined(PROC_FP_GUARDED)
+extern	struct pff_tab	Pgf_tab[];
+#endif	/* defined(PROC_FP_GUARDED) */
+
 
 /*
  * enter_file_info() -- enter file information
@@ -286,7 +289,7 @@ print_nm(lf)
 
 #if	defined(PROC_FP_GUARDED)
 	if (extra > 1)
-	    putchar(`,');
+	    putchar(',');
 	if (lf->guardflags) {
 	    struct pff_tab *tp;
 	    long gf;
@@ -391,7 +394,7 @@ process_kqueue(pid, fd)
 	    (void) fprintf(stderr,
 		"      too few bytes; expected %ld, got %d\n",
 		sizeof(kq), nb);
-	    Exit(1);
+	    Error();
 	}
 /*
  * Enter the kernel queue file information.
@@ -453,8 +456,8 @@ process_pipe_common(pi)
 		(SZOFFTYPE)pi->pipeinfo.pipe_stat.vst_size);
 	}
 }
-	
-	
+
+
 void
 process_pipe(pid, fd)
 	int pid;			/* PID */
@@ -476,7 +479,7 @@ process_pipe(pid, fd)
 	    (void) fprintf(stderr,
 		"      too few bytes; expected %ld, got %d\n",
 	       sizeof(pi), nb);
-	    Exit(1);
+	    Error();
 	}
 
 	process_pipe_common(&pi);
@@ -505,7 +508,7 @@ process_fileport_pipe(pid, fp)
 	    (void) fprintf(stderr,
 		"      too few bytes; expected %ld, got %d\n",
 	       sizeof(pi), nb);
-	    Exit(1);
+	    Error();
 	}
 
 	process_pipe_common(&pi);
@@ -539,7 +542,7 @@ process_psem(pid, fd)
 	    (void) fprintf(stderr,
 		"      too few bytes; expected %ld, got %d\n",
 		sizeof(ps), nb);
-	    Exit(1);
+	    Error();
 	}
 /*
  * Enter the semaphore file information.
@@ -621,7 +624,7 @@ process_pshm(pid, fd)
 	    (void) fprintf(stderr,
 		"      too few bytes; expected %ld, got %d\n",
 		sizeof(ps), nb);
-	    Exit(1);
+	    Error();
 	}
 
 	process_pshm_common(&ps);
@@ -650,7 +653,7 @@ process_fileport_pshm(pid, fp)
 	    (void) fprintf(stderr,
 		"      too few bytes; expected %ld, got %d\n",
 		sizeof(ps), nb);
-	    Exit(1);
+	    Error();
 	}
 
 	process_pshm_common(&ps);
@@ -703,7 +706,7 @@ process_vnode(pid, fd)
 	    (void) fprintf(stderr,
 		"      too few bytes; expected %ld, got %d\n",
 		sizeof(vi), nb);
-	    Exit(1);
+	    Error();
 	}
 
 	process_vnode_common(&vi);
@@ -740,7 +743,7 @@ process_fileport_vnode(pid, fp)
 	    (void) fprintf(stderr,
 		"      too few bytes; expected %ld, got %d\n",
 		sizeof(vi), nb);
-	    Exit(1);
+	    Error();
 	}
 
 	process_vnode_common(&vi);
