@@ -33,7 +33,6 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: ddev.c,v 1.13 2000/12/04 14:24:15 abe Exp $";
 #endif
 
 #include "lsof.h"
@@ -52,7 +51,8 @@ static struct stat Dirsb;
  * Local definitions
  */
 
-#define	LIKE_NODE_TTL	"like device special "
+#define	LIKE_BLK_SPEC	"like block special"
+#define	LIKE_CHR_SPEC	"like character special"
 
 
 /*
@@ -84,14 +84,15 @@ clr_sect()
 
 
 /*
- * printchdevname() - print character device name
+ * printdevname() - print block and character device names
  */
 
 int
-printchdevname(dev, rdev, f)
+printdevname(dev, rdev, f, nty)
 	dev_t *dev;			/* device */
 	dev_t *rdev;			/* raw device */
 	int f;				/* 1 = follow with '\n' */
+	int nty;			/* node type: N_BLK or N_CHR */
 {
 	struct clone *c;
 	struct l_dev *dp;
@@ -103,19 +104,19 @@ printchdevname(dev, rdev, f)
 
 #if     defined(HASDCACHE)
 
-printchdevname_again:
+printdevname_again:
 
 #endif  /* defined(HASDCACHE) */
 
-	if (Clone && HAVECLONEMAJ && (*dev == DevDev)
-	&&  (major(*rdev) == CLONEMAJ))
+	if ((nty == N_CHR) && Clone && HAVECLONEMAJ && (*dev == DevDev)
+	&&  (GET_MAJ_DEV(*rdev) == CLONEMAJ))
 	{
 	    for (c = Clone; c; c = c->next) {
 		if (Devtp[c->dx].rdev == *rdev) {
 
 #if     defined(HASDCACHE)
 		    if (DCunsafe && !Devtp[c->dx].v && !vfy_dev(&Devtp[c->dx]))
-			goto printchdevname_again;
+			goto printdevname_again;
 #endif  /* defined(HASDCACHE) */
 
 		    safestrprt(Devtp[c->dx].name, stdout, f);
@@ -126,30 +127,47 @@ printchdevname_again:
 /*
  * Search device table for a full match.
  */
-	if ((dp = lkupdev(dev, rdev, 1, 0))) {
+
+#if	defined(HASBLKDEV)
+	if (nty == N_BLK)
+	    dp = lkupbdev(dev, rdev, 1, 0);
+	else
+#endif	/* defined(HASBLKDEV) */
+
+	dp = lkupdev(dev, rdev, 1, 0);
+	if (dp) {
 	    safestrprt(dp->name, stdout, f);
 	    return(1);
 	}
 /*
  * Search device table for a match without inode number and dev.
  */
-	if ((dp = lkupdev(&DevDev, rdev, 0, 0))) {
+
+#if	defined(HASBLKDEV)
+	if (nty == N_BLK)
+	    dp = lkupbdev(&DevDev, rdev, 0, 0);
+	else
+#endif	/* defined(HASBLKDEV) */
+
+	dp = lkupdev(&DevDev, rdev, 0, 0);
+	if (dp) {
 
 	/*
 	 * A raw device match was found.  Record it as a name column addition.
 	 */
-	    char *cp;
+	    char *cp, *ttl;
 	    int len;
 
-	    len = (int)(1 + strlen(LIKE_NODE_TTL) + strlen(dp->name) + 1);
+	    ttl = (nty == N_BLK) ? LIKE_BLK_SPEC : LIKE_CHR_SPEC;
+	    len = (int)(1 + strlen(ttl) + 1 + strlen(dp->name) + 1);
 	    if (!(cp = (char *)malloc((MALLOC_S)(len + 1)))) {
-		(void) fprintf(stderr, "%s: no nma space for: (%s%s)\n",
-		    Pn, LIKE_NODE_TTL, dp->name);
-		Exit(1);
+		(void) fprintf(stderr, "%s: no nma space for: (%s %s)\n",
+		    Pn, ttl, dp->name);
+		Error();
 	    }
-	    (void) snpf(cp, len + 1, "(%s%s)", LIKE_NODE_TTL, dp->name);
+	    (void) snpf(cp, len + 1, "(%s %s)", ttl, dp->name);
 	    (void) add_nma(cp, len);
-	    (void) free((MALLOC_P *)cp);
+	    (void) free((FREE_P *)cp);
 	    return(0);
 	}
 
@@ -160,7 +178,7 @@ printchdevname_again:
  */
 	if (DCunsafe) {
 	    (void) rereaddev();
-	    goto printchdevname_again;
+	    goto printdevname_again;
 	}
 #endif  /* defined(HASDCACHE) */
 
@@ -240,7 +258,7 @@ readdev(skip)
 	    {
 		(void) fprintf(stderr, "%s: no space for: ", Pn);
 		safestrprt(Dstk[Dstkx], stderr, 1);
-		Exit(1);
+		Error();
 	    }
 	    (void) free((FREE_P *)Dstk[Dstkx]);
 	    Dstk[Dstkx] = (char *)NULL;
@@ -263,7 +281,7 @@ readdev(skip)
 		    (void) fprintf(stderr, "%s: no space for: ", Pn);
 		    safestrprt(path, stderr, 0);
 		    safestrprt(dp->d_name, stderr, 1);
-		    Exit(1);
+		    Error();
 		}
 
 #if	defined(USE_STAT)
@@ -312,27 +330,29 @@ readdev(skip)
 			if (!Devtp) {
 			    (void) fprintf(stderr,
 				"%s: no space for character device\n", Pn);
-			    Exit(1);
+			    Error();
 			}
 		    }
-		    Devtp[i].inode = sb.st_ino;
+		    Devtp[i].inode = (INODETYPE)sb.st_ino;
 		    if (!(Devtp[i].name = mkstrcpy(fp, (MALLOC_S *)NULL))) {
 			(void) fprintf(stderr, "%s: no space for: ", Pn);
 			safestrprt(fp, stderr, 1);
-			Exit(1);
+			Error();
 		    }
 		    Devtp[i].rdev = sb.st_rdev;
 		    Devtp[i].v = 0;
 		/*
 		 * Save clone device location.
 		 */
-		    if (HAVECLONEMAJ && major(Devtp[i].rdev) == CLONEMAJ) {
+		    if (HAVECLONEMAJ
+		    &&   GET_MAJ_DEV(Devtp[i].rdev) == CLONEMAJ)
+		    {
 			if (!(c = (struct clone *)malloc(sizeof(struct clone))))
 			{
 			    (void) fprintf(stderr,
 				"%s: no space for clone device: ", Pn);
 			    safestrprt(fp, stderr, 1);
-			    Exit(1);
+			    Error();
 			}
 			c->dx = i;
 			c->next = Clone;
@@ -359,10 +379,10 @@ readdev(skip)
 			if (!BDevtp) {
 			    (void) fprintf(stderr,
 				"%s: no space for block device\n", Pn);
-				Exit(1);
+				Error();
 			}
 		    }
-		    BDevtp[j].inode = sb.st_ino;
+		    BDevtp[j].inode = (INODETYPE)sb.st_ino;
 		    BDevtp[j].name = fp;
 		    fp = (char *)NULL;
 		    BDevtp[j].rdev = sb.st_rdev;
@@ -404,7 +424,7 @@ readdev(skip)
 	    {
 		(void) fprintf(stderr,
 		    "%s: no space for block device sort pointers\n", Pn);
-		Exit(1);
+		Error();
 	    }
 	    for (j = 0; j < BNdev; j++) {
 		BSdev[j] = &BDevtp[j];
@@ -434,7 +454,7 @@ readdev(skip)
 	    {
 		(void) fprintf(stderr,
 		    "%s: no space for character device sort pointers\n", Pn);
-		Exit(1);
+		Error();
 	    }
 	    for (i = 0; i < Ndev; i++) {
 		Sdev[i] = &Devtp[i];
@@ -444,7 +464,7 @@ readdev(skip)
 	    Ndev = rmdupdev(&Sdev, Ndev, "char");
 	} else {
 	    (void) fprintf(stderr, "%s: no character devices found\n", Pn);
-	    Exit(1);
+	    Error();
 	}
 
 #if	defined(HASDCACHE)
@@ -498,7 +518,7 @@ readdev(skip)
  * and/or their "/dev/fd" filesystem.  The problem is that when applied to
  * "/dev/fs", getdirentries() returns the wrong size, which can cause readdir()
  * to run off the end of it's internal buffer and return bogus file names.
- * 
+ *
  * The changes from the original NetBSD file are:
  *
  * - uses of the field dd_flags in the DIR structure have been removed since
@@ -699,7 +719,7 @@ rmdupdev(dp, n, nm)
 	{
 	    (void) fprintf(stderr, "%s: can't realloc %s device pointers\n",
 		Pn, nm);
-	    Exit(1);
+	    Error();
 	}
 	return(j);
 }
@@ -728,7 +748,7 @@ vfy_dev(dp)
 #endif	/* defined(USE_STAT) */
 
 	||  dp->rdev != sb.st_rdev
-	||  dp->inode != sb.st_ino) {
+	||  dp->inode != (INODETYPE)sb.st_ino) {
 	   (void) rereaddev();
 	   return(0);
 	}
