@@ -33,7 +33,6 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dproc.c,v 1.19 2001/02/13 13:52:18 abe Exp $";
 #endif
 
 #include "lsof.h"
@@ -66,6 +65,13 @@ _PROTOTYPE(static void process_text,(KA_T tp));
 _PROTOTYPE(static void read_proc,(void));
 
 
+/*
+ * Local defintions
+ */
+
+#define	PAPSINCR	1024		/* Pa and Ps table increment */
+#define	PAPSINIT	512		/* Pa and Ps table initial size */
+
 
 /*
  * Local static values
@@ -124,7 +130,7 @@ enter_vn_text(va, n)
 	    if (!Vp) {
 		(void) fprintf(stderr, "%s: no txt ptr space, PID %d\n",
 		    Pn, Lp->pid);
-		Exit(1);
+		Error();
 	    }
 	}
 /*
@@ -278,7 +284,7 @@ gather_proc_info()
 				(void) fprintf(stderr,
 				    "%s: PID %d, no file * space\n",
 				    Pn, Lp->pid);
-				Exit(1);
+				Error();
 			    }
 			    nufb = b;
 			}
@@ -299,7 +305,7 @@ gather_proc_info()
 				    (void) fprintf(stderr,
 					"%s: PID %d: no file flags space\n",
 					Pn, Lp->pid);
-				    Exit(1);
+				    Error();
 				}
 				pofb = b;
 			    }
@@ -438,7 +444,7 @@ get_kernel_access()
 	    if (!(Nmlst = get_nlist_path(1))) {
 		(void) fprintf(stderr, "%s: can't get kernel name list path\n",
 		    Pn);
-		Exit(1);
+		Error();
 	    }
 	}
 #endif	/* DUV<40000 */
@@ -455,7 +461,7 @@ get_kernel_access()
  * See if the non-KMEM memory file is readable.
  */
 	if (Memory && !is_readable(Memory, 1))
-	    Exit(1);
+	    Error();
 #endif	/* defined(WILLDROPGID) */
 
 /*
@@ -464,7 +470,7 @@ get_kernel_access()
 	if ((Kd = open(Memory ? Memory : KMEM, O_RDONLY, 0)) < 0) {
 	    (void) fprintf(stderr, "%s: can't open %s: %s\n", Pn,
 		Memory ? Memory : KMEM, sys_errlist[errno]);
-	    Exit(1);
+	    Error();
 	}
 
 #if	defined(WILLDROPGID)
@@ -478,7 +484,7 @@ get_kernel_access()
  * See if the name list file is readable.
  */
 	if (Nmlst && !is_readable(Nmlst, 1))
-	    Exit(1);
+	    Error();
 #endif	/* defined(WILLDROPGID) */
 
 /*
@@ -491,13 +497,13 @@ get_kernel_access()
 	    rv = knlist(Nl);
 	else
 #endif	/* DUV>=40000 */
-	
+
             rv = nlist(Nmlst, Nl);
 	if (rv == -1) {
 	    (void) fprintf(stderr,
 		"%s: can't read kernel name list from %s: %s\n",
-		Pn, Nmlst ? Nmlst : "kernel", strerror(errno));
-	    Exit(1);
+		Pn, Nmlst ? Nmlst : "knlist(3)", strerror(errno));
+	    Error();
 	}
 
 #if	DUV<30000
@@ -514,12 +520,12 @@ get_kernel_access()
 
 	{
 	    (void) fprintf(stderr, "%s: can't read proc table info\n", Pn);
-	    Exit(1);
+	    Error();
 	}
 	if (get_Nl_value("vnmaxp", Drive_Nl, &v) < 0 || !v
 	||  kread(v, (char *)&Vnmxp, sizeof(Vnmxp))) {
 	    (void) fprintf(stderr, "%s: can't determine vnode length\n", Pn);
-	    Exit(1);
+	    Error();
 	}
 	if (get_Nl_value("cldev", Drive_Nl, &v) < 0 || !v
 	||  kread(v, (char *)&dev, sizeof(dev))) {
@@ -528,7 +534,7 @@ get_kernel_access()
 		    Pn);
 	    HaveCloneMaj = 0;
 	} else {
-	    CloneMaj = major(dev);
+	    CloneMaj = GET_MAJ_DEV(dev);
 	    HaveCloneMaj = 1;
 	}
 }
@@ -558,7 +564,7 @@ get_nlist_path(ap)
 	    if (rv < 0) {
 		(void) fprintf(stderr, "%s: can't get booted file name: %s\n",
 		    Pn, strerror(errno));
-		Exit(1);
+		Error();
 	    }
 	    return((char *)NULL);
 	}
@@ -593,7 +599,7 @@ get_nlist_path(ap)
 	    (void) fprintf(stderr,
 		"%s: can't allocate %d bytes for boot file path: %s\n",
 		Pn, len, ba);
-	    Exit(1);
+	    Error();
 	}
 	(void) snpf(ps, len, "%s", ba);
 	return(ps);
@@ -789,6 +795,7 @@ process_text(tp)
 static void
 read_proc()
 {
+	static int ap = 0;
 	MALLOC_S len;
 	struct proc *p;
 	KA_T pa;
@@ -805,27 +812,28 @@ read_proc()
 	 */
 	    if (Np < 1) {
 		(void) fprintf(stderr, "%s: proc table has no entries\n", Pn);
-		Exit(1);
+		Error();
 	    }
-	    len = (MALLOC_S)(Np * sizeof(struct proc));
+	    len = (MALLOC_S)(PAPSINIT * sizeof(struct proc));
 	    if (!(Ps = (struct proc *)malloc(len))) {
 		(void) fprintf(stderr, "%s: no proc table space (%d bytes)\n",
 		    Pn, len);
-		Exit(1);
+		Error();
 	    }
 
 #if	DUV>=30000
 	/*
 	 * Allocate kernel proc address table space.
 	 */
-	    len = (MALLOC_S)(Np * sizeof(KA_T));
+	    len = (MALLOC_S)(PAPSINIT * sizeof(KA_T));
 	    if (!(Pa = (KA_T *)malloc(len))) {
 		(void) fprintf(stderr,
 		    "%s: no proc address table space (%d bytes)\n", Pn, len);
-		Exit(1);
+		Error();
 	    }
 #endif	/* DUV>=30000 */
 
+	    ap = PAPSINIT;
 	}
 /*
  * Try to read the proc structure table PROCTRYLM times.
@@ -834,6 +842,32 @@ read_proc()
 	for (try = 0; try < PROCTRYLM; try++) {
 	    for (p = Ps, Psn = px = 0; px < Np; px++) {
 
+	    /*
+	     * Insure Ps and Psa space.
+	     */
+		if (Psn >= ap) {
+		    ap += PAPSINCR;
+		    len = (MALLOC_S)(ap * sizeof(struct proc));
+		    if (!(Ps = (struct proc *)realloc((MALLOC_P *)Ps, len))) {
+			(void) fprintf(stderr,
+			    "%s: no more proc table space (%d bytes)\n",
+			    Pn, len);
+			Error();
+		    }
+		    p = &Ps[Psn];
+
+#if	DUV>=30000
+		    len = (MALLOC_S)(ap * sizeof(KA_T));
+		    if (!(Pa = (KA_T *)realloc((MALLOC_P *)Pa, len))) {
+			(void) fprintf(stderr,
+			    "%s: no more proc address table space (%d bytes)\n",
+			    Pn, len);
+			Error();
+		    }
+#endif	/* DUV>=30000 */
+
+		}
+
 #if     DUV<30000
 		pa = Kp + (KA_T)(px * sizeof(struct proc));
 		if (kread(pa, (char *)p, sizeof(struct proc)))
@@ -864,7 +898,7 @@ read_proc()
  */
 	if (try >= PROCTRYLM) {
 	    (void) fprintf(stderr, "%s: can't read proc table\n", Pn);
-	    Exit(1);
+	    Error();
 	}
 	if (Psn < Np && !RptTm) {
 
@@ -872,12 +906,13 @@ read_proc()
 	 * Reduce the local proc structure tables to their minimum if
 	 * not in repeat mode.
 	 */
+	    ap = Psn;
 	    len = (MALLOC_S)(Psn * sizeof(struct proc));
 	    if (!(Ps = (struct proc *)realloc((MALLOC_P *)Ps, len))) {
 		(void) fprintf(stderr,
 		    "%s: can't reduce proc table to %d bytes\n",
 		    Pn, len);
-		Exit(1);
+		Error();
 	    }
 
 #if	DUV>=30000
@@ -886,7 +921,7 @@ read_proc()
 		(void) fprintf(stderr,
 		    "%s: can't reduce proc address table to %d bytes\n",
 		    Pn, len);
-		Exit(1);
+		Error();
 	    }
 #endif	/* DUV>=30000 */
 
@@ -934,7 +969,7 @@ vpo2vp(vpo)
 	if (!ka || kread(ka, (char *)&bfa, sizeof(bfa)))
 	    return(ka);
 	return((KA_T)bfa.vp);
-} 
+}
 #endif	/* DUV>=50000 */
 
 
@@ -1169,7 +1204,7 @@ ncache_ckrootid(na, id)
 	    if (!ic) {
 		(void) fprintf(stderr,
 		    "%s: no space for root node VPID table\n", Pn);
-		Exit(1);
+		Error();
 	    }
 	    nia += 10;
 	}
@@ -1239,7 +1274,7 @@ ncache_isroot(na, cp)
 		buf[len++] = '/';
 	    (void) strcpy(&buf[len], cp);
 	    if (statsafely(buf, &sb) != 0
-	    ||  (unsigned long)sb.st_ino != Lf->inode)
+	    ||  (INODETYPE)sb.st_ino != Lf->inode)
 		return(0);
 	}
 /*
@@ -1256,7 +1291,7 @@ ncache_isroot(na, cp)
 	    if (!nc) {
 		(void) fprintf(stderr,
 		    "%s: no space for root node address table\n", Pn);
-		Exit(1);
+		Error();
 	    }
 	    nca += 10;
 	}
@@ -1339,7 +1374,7 @@ ncache_load()
 		(void) fprintf(stderr,
 		    "%s: can't allocate %d bytes for processor addresses\n",
 			Pn, len);
-		Exit(1);
+		Error();
 	    }
 	    if (kread(ka, (char *)pp, len)) {
 		if (!Fwarn)
@@ -1387,7 +1422,7 @@ ncache_load()
 		(void) fprintf(stderr,
 		    "%s: no space for %d namecache entries (%d bytes)\n",
 		    Pn, ncpc * ncpus, len);
-		Exit(1);
+		Error();
 	    }
 	} else {
 
@@ -1434,7 +1469,7 @@ ncache_load()
 		(void) fprintf(stderr,
 		    "%s: no space for %d byte name cache hash buckets\n",
 		    Pn, (int)(i * sizeof(struct l_nch *)));
-	    Exit(1);
+	    Error();
 	}
 /*
  * Assign hash pointers to the accumulated namecache entries.
@@ -1467,7 +1502,7 @@ ncache_load()
 	 */
 	    if (!(lp = (struct l_nch *)malloc(sizeof(struct l_nch)))) {
 		(void) fprintf(stderr, "%s: can't allocate l_nch entry\n", Pn);
-		Exit(1);
+		Error();
 	    }
 	    lp->nc = &nc[i];
 	    lp->nxt = Nchash[h];
@@ -1520,9 +1555,9 @@ ncache_lookup(buf, blen, fp)
 	    for (mtp = readmnt(); mtp; mtp = mtp->next) {
 		if (!mtp->dir || !mtp->inode)
 		    continue;
-		if (Lf->dev == mtp->dev
-		&&  (unsigned long)mtp->inode == Lf->inode
-		&&  strcmp(mtp->dir, Lf->fsdir) == 0)
+		if ((Lf->dev == mtp->dev)
+		&&  (mtp->inode == Lf->inode)
+		&&  (strcmp(mtp->dir, Lf->fsdir) == 0))
 		    return(cp);
 	    }
 	    return((char *)NULL);
