@@ -32,7 +32,6 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.16 2001/06/19 13:29:51 abe Exp $";
 #endif
 
 #if	defined(HPUXKERNBITS) && HPUXKERNBITS>=64
@@ -80,6 +79,9 @@ static int
 islocked(lp)
 	KA_T lp;			/* local locklist struct pointer */
 {
+	static int ety = -1;
+	static unsigned int ei = 0;
+	static SZOFFTYPE el = 0;
 	int l;
 	struct locklist ll;
 	KA_T llf, llp;
@@ -87,6 +89,27 @@ islocked(lp)
 	if (!(llf = (KA_T)lp))
 	    return((int)' ');
 	llp = llf;
+/*
+ * Compute the end test value the first time through.
+ */
+
+	if (ety == -1) {
+
+# if	HPUXV<1020
+	    ety = 0;
+	    ei = 0x7fffffff;
+# else	/* HPUXV>=1020 */
+	    if (sizeof(ll.ll_end) == 4) {
+		ety = 0;
+		ei = 0x80000000;
+	    } else {
+		ety = 1;
+		el = 0x10000000000ll;
+	    }
+# endif	/* HPUXV<1020 */
+
+	}
+
 /*
  * Search the locklist chain for this process.
  */
@@ -105,22 +128,19 @@ islocked(lp)
 # endif	/* HPUXV<1010 */
 
 		continue;
-	    if (ll.ll_start == 0
-
-# if	HPUXV<1030
-		&& ll.ll_end == 0x7fffffff
-# else	/* HPUXV>=1030 */
-#  if	defined(HPUXKERNBITS) && HPUXKERNBITS>=64
-		&& (ll.ll_end == 0x80000000 || ll.ll_end == 0x10000000000)
-#  else	/* !defined(HPUXKERNBITS) || HPUXKERNBITS<64 */
-		&& ll.ll_end == 0x80000000
-#  endif	/* defined(HPUXKERNBITS) && HPUXKERNBITS>=64 */
-# endif	/* HPUXV<1030 */
-
-	    )
-		l = 1;
-	    else
-		l = 0;
+	    l = 0;
+	    if (ll.ll_start == 0) {
+		switch (ety) {
+		case 0:
+		    if (ll.ll_end == ei)
+			l = 1;
+		break;
+		case 1:
+		    if (ll.ll_end == el)
+			l = 1;
+		break;
+		}
+	    }
 	    if (ll.ll_type == F_WRLCK)
 		return((int)(l ? 'W' : 'w'));
 	    else if (ll.ll_type == F_RDLCK)
@@ -232,7 +252,7 @@ getnodety(v)
 	/*
 	 * Caution: this AFS test should be the last one.
 	 */
-	
+
 	else if (avops) {
 	    if (avops == (unsigned long)v->v_op)
 		return(N_AFS);
@@ -351,7 +371,7 @@ process_node(va)
 #endif	/* defined(HAS_AFS) */
 
 		);
-		Exit(1);
+		Error();
 	    }
 	}
 	if (readvnode(va, v)) {
@@ -469,7 +489,7 @@ process_node(va)
 	    if (!v->v_data || readrnode((KA_T)v->v_data, &r)) {
 		(void) snpf(Namech, Namechl,
 		    "vnode at %s: can't read rnode (%s)",
-		    print_kptr(va, tbuf, sizeof(tbuf)), 
+		    print_kptr(va, tbuf, sizeof(tbuf)),
 		    print_kptr((KA_T)v->v_data, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
@@ -479,7 +499,7 @@ process_node(va)
 
 #if	HPUXV>=1000
 	case N_SPEC:
-	    if (v->v_type == VCHR) {
+	    if ((v->v_type == VBLK) || (v->v_type == VCHR)) {
 		if (!v->v_data || readsnode((KA_T)v->v_data, &s)) {
 		    (void) snpf(Namech, Namechl,
 			"vnode at %s: can't read snode(%s)",
@@ -572,17 +592,15 @@ process_node(va)
 
 	case N_REGLR:
 	default:
-	    if (v->v_type != VBLK) {
-		if (!v->v_data || readinode((KA_T)v->v_data, &i)) {
-		    (void) snpf(Namech, Namechl,
-			"vnode at %s: can't read inode (%s)",
-			print_kptr(va, tbuf, sizeof(tbuf)),
-			print_kptr((KA_T)v->v_data, (char *)NULL, 0));
-		    enter_nm(Namech);
-		    return;
-		}
-		ins = 1;
+	    if (!v->v_data || readinode((KA_T)v->v_data, &i)) {
+		(void) snpf(Namech, Namechl,
+		    "vnode at %s: can't read inode (%s)",
+		    print_kptr(va, tbuf, sizeof(tbuf)),
+		    print_kptr((KA_T)v->v_data, (char *)NULL, 0));
+		enter_nm(Namech);
+		return;
 	    }
+	    ins = 1;
 
 #if	HPUXV>=900 && HPUXV<1000
 	    if (v->v_type == VFIFO)
@@ -653,7 +671,7 @@ process_node(va)
 		dev = i.i_dev;
 		devs = 1;
 	    }
-	    if ((v->v_type == VCHR) || (v->v_type == N_BLK)) {
+	    if ((v->v_type == VBLK) || (v->v_type == VCHR)) {
 		rdev = v->v_rdev;
 		rdevs = 1;
 	    }
@@ -678,22 +696,22 @@ process_node(va)
 #if	defined(HAS_AFS)
 	case N_AFS:
 	    if (an.ino_st) {
-		Lf->inode = an.inode;
+		Lf->inode = (INODETYPE)an.inode;
 		Lf->inp_ty = 1;
 	    }
 	    break;
 #endif	/* defined(HAS_AFS) */
 
 	case N_MVFS:
-	    Lf->inode = (unsigned long)m.m_ino;
+	    Lf->inode = (INODETYPE)m.m_ino;
 	    Lf->inp_ty = 1;
 	    break;
 	case N_NFS:
 
 #if	HPUXV<1030
-	    Lf->inode = (unsigned long)r.r_nfsattr.na_nodeid;
+	    Lf->inode = (INODETYPE)r.r_nfsattr.na_nodeid;
 #else	/* HPUXV>=1030 */
-	    Lf->inode = (unsigned long)r.r_attr.va_nodeid;
+	    Lf->inode = (INODETYPE)r.r_attr.va_nodeid;
 #endif	/* HPUXV<1030 */
 
 	    Lf->inp_ty = 1;
@@ -701,16 +719,16 @@ process_node(va)
 
 #if	HPUXV>=1000
 	case N_CDFS:
-	    Lf->inode = (unsigned long)c.cd_num;
+	    Lf->inode = (INODETYPE)c.cd_num;
 	    Lf->inp_ty = 1;
 	    break;
 	case N_FIFO:
 	case N_PIPE:
 	    if (vats) {
-		Lf->inode = (unsigned long)vat.va_nodeid;
+		Lf->inode = (INODETYPE)vat.va_nodeid;
 		Lf->inp_ty = 1;
 	    } else {
-		Lf->inode = (unsigned long)v->v_nodeid;
+		Lf->inode = (INODETYPE)v->v_nodeid;
 		Lf->inp_ty = 1;
 	    }
 	    break;
@@ -727,7 +745,7 @@ process_node(va)
 
 # if	HPUXV>=900
 	    if (rns) {
-		Lf->inode = (unsigned long)r.r_nfsattr.na_nodeid;
+		Lf->inode = (INODETYPE)r.r_nfsattr.na_nodeid;
 		Lf->inp_ty = 1;
 		break;
 	    }
@@ -736,10 +754,11 @@ process_node(va)
 
 #endif	/* HPUXV<1000 */
 
+	case N_BLK:
 	case N_REGLR:
 	case N_SPEC:
 	    if (ins) {
-		Lf->inode = (unsigned long)i.i_number;
+		Lf->inode = (INODETYPE)i.i_number;
 		Lf->inp_ty = 1;
 	    }
 	}
@@ -883,6 +902,14 @@ process_node(va)
 		/* The location of the link count isn't known. */
 		break;
 	    case N_NFS:
+
+#if	HPUXV<1030
+		Lf->nlink = r.r_nfsattr.na_nlink;
+#else	/* HPUXV>=1030 */
+		Lf->nlink = r.r_attr.va_nlink;
+#endif	/* HPUXV<1030 */
+
+		Lf->nlink_def = 1;
 		break;
 
 # if	HPUXV>=1000
@@ -991,11 +1018,7 @@ process_node(va)
 	    }
 	    break;
 	default:
-	    if (type > 9999)
-		(void) snpf(Lf->type, sizeof(Lf->type), "*%03d", type % 1000);
-	    else
-		(void) snpf(Lf->type, sizeof(Lf->type), "%4d", type);
-	    (void) snpf(Namech, Namechl, "unknown type");
+	    (void) snpf(Lf->type, sizeof(Lf->type), "%04o", (type & 0xfff));
 	    ty = (char *)NULL;
 	}
 	if (ty)
