@@ -29,13 +29,6 @@
  * 4. This notice may not be removed or altered.
  */
 
-#ifndef	lint
-static char copyright[] =
-"@(#) Copyright 1997 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dmnt.c,v 1.19 2012/04/10 16:39:50 abe Exp $";
-#endif
-
-
 #include "lsof.h"
 
 
@@ -108,7 +101,7 @@ cvtoe(os)
 	    (void) fprintf(stderr,
 		"%s: can't allocate %d bytes for octal-escaping.\n",
 		Pn, ol + 1);
-	    Exit(1);
+	    Error();
 	}
 /*
  * Copy the string, replacing octal-escaped characters as they are found.
@@ -159,7 +152,7 @@ cvtoe(os)
 		    (void) fprintf(stderr,
 			"%s: can't realloc %d bytes for octal-escaping.\n",
 			Pn, cl + 1);
-		    Exit(1);
+		    Error();
 		}
 	    }
 	/*
@@ -301,7 +294,7 @@ getmntdev(dn, dnl, s, ss)
 			(void) fprintf(stderr,
 			    "%s: no space for mount supplement hash buckets\n",
 			    Pn);
-			Exit(1);
+			Error();
 		    }
 		}
 		h = hash_mnt(path);
@@ -331,13 +324,13 @@ getmntdev(dn, dnl, s, ss)
 		    (void) fprintf(stderr,
 			"%s: no space for mount supplement entry: %d \"%s\"\n",
 			Pn, ln, buf);
-		    Exit(1);
+		    Error();
 		}
 		if (!(mpn->dn = (char *)malloc(sz + 1))) {
 		    (void) fprintf(stderr,
 			"%s: no space for mount supplement path: %d \"%s\"\n",
 			Pn, ln, buf);
-		    Exit(1);
+		    Error();
 		}
 		(void) strcpy(mpn->dn, path);
 		mpn->dnl = sz;
@@ -378,7 +371,7 @@ getmntdev(dn, dnl, s, ss)
 	h = hash_mnt(dn);
 	for (mp = MSHash[h]; mp; mp = mp->next) {
 	    if ((dnl == mp->dnl) && !strcmp(dn, mp->dn)) {
-		memset((void *)s, 0, sizeof(struct stat));
+		zeromem((char *)s, sizeof(struct stat));
 		s->st_dev = mp->dev;
 		*ss |= SB_DEV;
 		return(1);
@@ -429,6 +422,7 @@ readmnt()
 	struct mounts *mp;
 	FILE *ms;
 	int nfs;
+	int mqueue;
 	struct stat sb;
 	static char *vbuf = (char *)NULL;
 	static size_t vsz = (size_t)0;
@@ -524,6 +518,12 @@ readmnt()
 	    if (*dn != '/')
 		continue;
 	    dnl = strlen(dn);
+
+	/*
+	 * Test Mqueue directory
+	 */
+	    mqueue = strcmp(fp[2], "mqueue");
+
 	/*
 	 * Test for duplicate and NFS directories.
 	 */
@@ -535,6 +535,8 @@ readmnt()
 		if ((nfs = strcasecmp(fp[2], "nfs3")))
 		    nfs = strcasecmp(fp[2], "nfs4");
 	    }
+	    if (!nfs && !HasNFS)
+		HasNFS = 1;
 	    if (mp) {
 
 	    /*
@@ -617,7 +619,7 @@ readmnt()
 		    (void) fprintf(stderr,
 			"%s: can't allocate mounts struct for: ", Pn);
 		    safestrprt(dn, stderr, 1);
-		    Exit(1);
+		    Error();
 	        }
 	    }
 	    mp->dir = dn;
@@ -633,8 +635,12 @@ readmnt()
 		mp->ty = N_NFS;
 		if (HasNFS < 2)
 		    HasNFS = 2;
-	    } else
+	    } else if (!mqueue) {
+		mp->ty = N_MQUEUE;
+		MqueueDev = mp->dev;
+	    } else {
 		mp->ty = N_REGLR;
+	    }
 
 #if	defined(HASMNTSUP)
 	/*
@@ -666,7 +672,7 @@ readmnt()
 		    (void) fprintf(stderr,
 			"%s: can't allocate space for: ", Pn);
 		    safestrprt(dn, stderr, 1);
-		    Exit(1);
+		    Error();
 		}
 		ignstat = 1;
 	    } else
