@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: dlsof.h,v 1.35 2004/12/30 18:42:24 abe Exp $
+ * $Id: dlsof.h,v 1.38 2006/03/28 21:54:08 abe Exp $
  */
 
 
@@ -41,6 +41,7 @@
 #include <stdlib.h>
 #include <dirent.h>
 #include <nlist.h>
+#include <paths.h>
 #include <setjmp.h>
 #include <signal.h>
 #include <string.h>
@@ -52,31 +53,38 @@
 
 #include <sys/filedesc.h>
 #include <sys/mbuf.h>
-  
+
 # if	defined(HAS_LWP_H)
 #include <sys/lwp.h>
 # endif	/* defined(HAS_LWP_H) */
 
 # if	(defined(OPENBSDV) && OPENBSDV>=3030) \
   ||	(defined(NETBSDV) && __NetBSD_Version__>=106060000)
+
+#  if	defined(OPENBSDV) ||  __NetBSD_Version__<399001100
 #define	_KERNEL
-#  if	defined(NETBSDV)
+#  endif	/* defined(OPENBSDV) ||  __NetBSD_Version__<399001100 */
+
+#  if	defined(NETBSDV) && __NetBSD_Version__<399001100
 struct buf;	/* dummy for function prototype in <sys/buf.h> */
 struct uio;	/* dummy for function prototype in <sys/buf.h> */
-#  endif	/* defined(NETBSDV) */
+#  endif	/* defined(NETBSDV && __NetBSD_Version__<399001100) */
+
 #include <sys/ucred.h>
 # endif	/* (defined(OPENBSDV) && OPENBSDV>=3030)
 	   || (defined(NETBSDV) && __NetBSD_Version__>=106060000) */
 
-# if	defined(NETBSDV)
+# if	defined(NETBSDV) && __NetBSD_Version__<399001100
 #include <sys/buf.h>
-# endif	/* defined(NETBSDV) */
+# endif	/* defined(NETBSDV) && __NetBSD_Version__<399001100 */
 
 # if	(defined(OPENBSDV) && OPENBSDV>=3030) \
-  ||	(defined(NETBSDV) && __NetBSD_Version__>=106060000)
+  ||	(defined(NETBSDV) && __NetBSD_Version__>=106060000 \
+  &&	__NetBSD_Version__<399001100)
 #undef	_KERNEL
-# endif	/* (defined(OPENBSDV) && OPENBSDV>=3030)
-		   || (defined(NETBSDV) && __NetBSD_Version__>=106060000) */
+# endif	/* (defined(OPENBSDV) && OPENBSDV>=3030) \
+	   || (defined(NETBSDV) && __NetBSD_Version__>=106060000 \
+	   &&  __NetBSD_Version__<399001100) */
 
 #define	NFS
 #define m_stat	mnt_stat
@@ -116,15 +124,16 @@ struct uio;	/* dummy for function prototype in <sys/buf.h> */
 #  if	(defined(OPENBSDV) && OPENBSDV<3030) \
    ||	(defined(NETBSDV) && __NetBSD_Version__<106060000)
 /*
- * As a terrible hack, the lsof Configure script extracts the netcred and
- * netexport structure definitions from <sys/mount.h> and places them in
- * "netexport.h".  The netexport structure is needed in the msdosfsmount
- * structure, defined in <msdosfs/msdosfsmount.h>.
+ * The netcred and netexport structures may be needed in the msdosfsmount
+ * structure, defined in <msdosfs/msdosfsmount.h>.  So as a terrible hack,
+ * the lsof Configure script extracts the netcred and netexport structure
+ * definitions from <sys/mount.h> and places them in "netexport.h".
  *
- * The netcred and netexport structures netcred should really be obtained
- * from <sys/mount.h>.  However they are hidden in <sys/mount.h> under _KERNEL,
- * and that can't be defined when including <sys/mount.h> without causing other
- * seemingly insurmountable #include problems.
+ * When needed, the netcred and netexport structures netcred should really
+ * be obtained from <sys/mount.h>.  However they are hidden in <sys/mount.h>
+ * under _KERNEL, and that sometimes can't be defined when including
+ * <sys/mount.h> without causing other seemingly insurmountable #include
+ * problems.
  *
  * THIS IS A TERRIBLE AND FRAGILE HACK!!!  It might break if the netexport or
  * netcred definitions change radically in <sys/mount.h>.
@@ -176,7 +185,7 @@ struct nameidata;	/* to satisfy a function prototype in msdosfsmount.h */
 #  endif	/* !defined(TF_ECN_PERMIT) */
 
 #  if	!defined(TF_RCVD_CE)
-#define	TF_RCVD_CE	0x00010000	/* send ECE in subsequent segs */ 
+#define	TF_RCVD_CE	0x00010000	/* send ECE in subsequent segs */
 #  endif	/* !defined(TF_RCVD_CE) */
 
 #  if	!defined(TF_SEND_CWR)
@@ -448,6 +457,10 @@ typedef	u_long		KA_T;
 #define FREE_P		MALLOC_P
 #define MALLOC_S	size_t
 
+# if	!defined(MAXSYSCMDL)
+#define	MAXSYSCMDL	MAXCOMLEN	/* max system command name length */
+# endif	/* !defined(MAXSYSCMDL) */
+
 # if	defined(N_UNIXV)
 #define	N_UNIX_TMP(x)	#x
 #define	N_UNIX_STR(x)	N_UNIX_TMP(x)
@@ -458,7 +471,8 @@ typedef	u_long		KA_T;
 #define	READLEN_T	int
 #define STRNCPY_L	size_t
 #define SWAP		"/dev/drum"
-#define	SZOFFTYPE	long long	/* size and offset internal storage
+#define	SZOFFTYPE	unsigned long long
+					/* size and offset internal storage
 					 * type */
 #define	SZOFFPSPEC	"ll"		/* SZOFFTYPE print specification
 					 * modifier */
@@ -490,7 +504,7 @@ struct mounts {
 					 * (symbolic links resolved) */
         dev_t dev;              	/* directory st_dev */
 	dev_t rdev;			/* directory st_rdev */
-	ino_t inode;			/* directory st_ino */
+	INODETYPE inode;		/* directory st_ino */
 	mode_t mode;			/* directory st_mode */
 	mode_t fs_mode;			/* file_system st_mode */
         struct mounts *next;    	/* forward link */
@@ -502,7 +516,7 @@ struct mounts {
 
 extern int Np;				/* number of kernel processes */
 
-# if	defined(HASKVMGETPROC2) 
+# if	defined(HASKVMGETPROC2)
 struct kinfo_proc2 *P;			/* local process table copy */
 # else	/* ! defined(HASKVMGETPROC2) */
 struct kinfo_proc *P;			/* local process table copy */
@@ -519,7 +533,7 @@ struct sfile {
 	u_short mode;			/* S_IFMT mode bits from stat() */
 	int type;			/* file type: 0 = file system
 				 	 *	      1 = regular file */
-	ino_t i;			/* inode number */
+	INODETYPE i;			/* inode number */
 	int f;				/* file found flag */
 	struct sfile *next;		/* forward link */
 
