@@ -32,7 +32,6 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dproc.c,v 1.16 2003/10/07 14:34:14 abe Exp abe $";
 #endif
 
 #include "lsof.h"
@@ -79,7 +78,7 @@ ckkv(d, er, ev, ea)
 	if (sysctl(m, 2, v, &l, NULL, 0) < 0) {
 	    (void) fprintf(stderr, "%s: CTL_KERN, KERN_OSRELEASE: %s\n",
 		Pn, strerror(errno));
-	    Exit(1);
+	    Error();
 	}
 /*
  * Warn if the actual and expected releases don't match.
@@ -131,7 +130,7 @@ enter_vn_text(va, n)
 	    if (!Vp) {
 		(void) fprintf(stderr, "%s: no txt ptr space, PID %d\n",
 		    Pn, Lp->pid);
-		Exit(1);
+		Error();
 	    }
 	}
 /*
@@ -187,12 +186,12 @@ gather_proc_info()
 	P = kvm_getproc2(Kd, KERN_PROC_ALL, 0, KVMPROCSZ2, &Np);
 #else	/* !defined(HASKVMGETPROC2) */
 	P = kvm_getprocs(Kd, KERN_PROC_ALL, 0, &Np);
-#endif	/* defined(HASKVMGETPROC2) &/
+#endif	/* defined(HASKVMGETPROC2) */
 
 	if (!P) {
 	    (void) fprintf(stderr, "%s: can't read process table: %s\n",
 		Pn, kvm_geterr(Kd));
-	    Exit(1);
+	    Error();
 	}
 /*
  * Examine proc structures and their associated information.
@@ -289,7 +288,7 @@ gather_proc_info()
 		if (!ofb) {
 		    (void) fprintf(stderr, "%s: PID %d, no file * space\n",
 			Pn, p->P_PID);
-		    Exit(1);
+		    Error();
 		}
 		ofbb = nb;
 	    }
@@ -307,7 +306,7 @@ gather_proc_info()
 		    if (!pof) {
 			(void) fprintf(stderr,
 			    "%s: PID %d, no file flag space\n", Pn, p->P_PID);
-		        Exit(1);
+		        Error();
 		    }
 		    pofb = nb;
 		}
@@ -377,7 +376,7 @@ get_kernel_access()
 	    if (!(Nmlst = get_nlist_path(1))) {
 		(void) fprintf(stderr,
 		    "%s: can't get kernel name list path\n", Pn);
-		Exit(1);
+		Error();
 	    }
 	}
 #endif	/* defined(N_UNIX) */
@@ -395,7 +394,7 @@ get_kernel_access()
  */
 	if ((Memory && !is_readable(Memory, 1))
 	||  (Nmlst && !is_readable(Nmlst, 1)))
-	    Exit(1);
+	    Error();
 #endif	/* defined(WILLDROPGID) */
 
 /*
@@ -403,17 +402,24 @@ get_kernel_access()
  */
 	if ((Kd = kvm_openfiles(Nmlst, Memory, NULL, O_RDONLY, NULL)) == NULL) {
 	    (void) fprintf(stderr,
-		"%s: kvm_openfiles (namelist=%s, core=%s): %s\n",
+		"%s: kvm_openfiles(execfile=%s, corefile=%s): %s\n",
 		Pn, Nmlst,
-		Memory  ? Memory  : "default",
+		Memory ? Memory :
+
+#if	defined(_PATH_MEM)
+				  _PATH_MEM,
+#else	/* !defined(_PATH_MEM) */
+				  "default",
+#endif	/* defined(_PATH_MEM) */
+
 		strerror(errno));
-	    Exit(1);
+	    Error();
 	}
 	(void) build_Nl(Drive_Nl);
 	if (kvm_nlist(Kd, Nl) < 0) {
 	    (void) fprintf(stderr, "%s: can't read namelist from %s\n",
 		Pn, Nmlst);
-	    Exit(1);
+	    Error();
 	}
 
 #if	defined(WILLDROPGID)
@@ -459,7 +465,7 @@ get_nlist_path(ap)
 		(void) fprintf(stderr,
 		    "%s: can't allocate %d bytes for boot file path: %s\n",
 		    Pn, bfl, bf);
-		Exit(1);
+		Error();
 	    }
 	    (void) snpf(bfc, bfl, "%s", bf);
 	    return(bfc);
