@@ -32,7 +32,6 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1995 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.14 2000/12/04 14:32:49 abe Exp $";
 #endif
 
 
@@ -71,7 +70,7 @@ finddev_again:
  */
 	if (stream && Clone) {
 	    for (c = Clone; c; c = c->next) {
-		if (major(*rdev) == minor(Devtp[c->dx].rdev)) {
+		if (GET_MAJ_DEV(*rdev) == GET_MIN_DEV(Devtp[c->dx].rdev)) {
 
 #if	defined(HASDCACHE)
 		    if (DCunsafe && !Devtp[c->dx].v && !vfy_dev(&Devtp[c->dx]))
@@ -156,7 +155,7 @@ process_node(na)
  * Identify the node type.
  */
 	if (HaveSockdev && (i.i_ftype & IFMT) == IFCHR
-	&&  major(i.i_rdev) == Sockdev)
+	&&  GET_MAJ_DEV(i.i_rdev) == Sockdev)
 	{
 
 	/*
@@ -230,6 +229,9 @@ process_node(na)
 	    case IFMPC:
 		Ntype = N_MPC;
 		break;
+	    case IFNAM:
+		Ntype = N_NM;
+		break;
 	    }
 	}
 /*
@@ -285,11 +287,13 @@ process_node(na)
  */
 	switch (Ntype) {
 	case N_BLK:
-	    Lf->dev = i.i_rdev;
-	    Lf->dev_def = 1;
+	    Lf->dev = i.i_dev;
+	    Lf->rdev = i.i_rdev;
+	    Lf->dev_def = Lf->rdev_def = 1;
 	    break;
 	case N_FIFO:
 	case N_HSFS:
+	case N_NM:
 	case N_REGLR:
 
 #if	OSRV>=500
@@ -301,7 +305,7 @@ process_node(na)
 	    Lf->dev_def = 1;
 	    break;
 	case N_CHR:
-	    Lf->dev = DevDev;
+	    Lf->dev = i.i_dev;
 	    Lf->rdev = i.i_rdev;
 	    Lf->dev_def = Lf->rdev_def = 1;
 	    if (i.i_sptr) {
@@ -309,7 +313,7 @@ process_node(na)
 	    /*
 	     * Namech may be:
 	     *    /dev/* name if it exists for i.i_rdev;
-	     *    cdevsw[].d_name if it exists for major(i.i_rdev);
+	     *    cdevsw[].d_name if it exists for GET_MAJ_DEV(i.i_rdev);
 	     *    "STR:" otherwise.
 	     */
 		(void) snpf(Namech, Namechl, "STR:");
@@ -321,14 +325,15 @@ process_node(na)
 		    k += strlen(dp->name);
 		    if ((cp = strrchr(dp->name, '/')))
 			cp++;
-		} else if ((j = major(i.i_rdev)) < Cdevcnt && (cp = Cdevsw[j]))
+		} else if ((j = GET_MAJ_DEV(i.i_rdev))
+			<  Cdevcnt && (cp = Cdevsw[j]))
 		{
 		    (void) snpf(Namech, Namechl, "%s", cp);
 		    k += strlen(cp);
 		}
 	    /*
 	     * Get the module names of all queue elements of the stream's
-	     * sd_wrq queue.  Skip module names that end in "head", 
+	     * sd_wrq queue.  Skip module names that end in "head",
 	     * match the last component of the /dev name, or match the
 	     * cdevsw[].d_name.
 	     */
@@ -461,7 +466,7 @@ process_node(na)
 			 if (udptm && !Lf->nma)
 			     (void) udp_tm(udp.ud_ftime);
 		    }
-		    if (i.i_number == 0)
+		    if (!i.i_number)
 			Lf->inp_ty = 2;
 		}
 	    } else {
@@ -474,9 +479,9 @@ process_node(na)
 		    Ntype = N_CHR;
 		    if (!finddev(&i.i_dev, &i.i_rdev, 0)
 		    &&  HaveEventMajor
-		    &&  major(i.i_rdev) == EventMajor)
+		    &&  GET_MAJ_DEV(i.i_rdev) == EventMajor)
 			(void) snpf(Namech, Namechl,
-			    "clone %d:/dev/event", minor(i.i_rdev));
+			    "clone %d:/dev/event", GET_MIN_DEV(i.i_rdev));
 		}
 	    }
 	    break;
@@ -485,12 +490,16 @@ process_node(na)
 	case N_NFS:
 
 #if	OSRV<500
-	    Lf->dev = (dev_t)_makedev(~major(i.i_dev), minor(i.i_dev));
+	    Lf->dev = (dev_t)_makedev(~GET_MAJ_DEV(i.i_dev),
+				      GET_MIN_DEV(i.i_dev));
+	    Lf->rdev = (dev_t)_makedev(~GET_MAJ_DEV(i.i_rdev),
+				       GET_MIN_DEV(i.i_rdev));
 #else	/* OSRV>=500 */
 	    Lf->dev = i.i_dev;
+	    Lf->rdev = i.i_rdev;
 #endif	/* OSRV<500 */
 
-	    Lf->dev_def = 1;
+	    Lf->dev_def = Lf->rdev_def = 1;
 	    break;
 #endif	/* defined(HAS_NFS) */
 
@@ -534,9 +543,11 @@ process_node(na)
 	    break;
 #endif	/* defined(HAS_NFS) */
 
+	case N_BLK:
 	case N_CHR:
 	case N_COM:
 	case N_FIFO:
+	case N_NM:
 	case N_REGLR:
 
 #if	OSRV>=500
@@ -633,6 +644,17 @@ process_node(na)
 	case IFMPB:
 	    tn = "MPB";
 	    break;
+	case IFNAM:
+	    if (i.i_rdev == S_INSEM)
+		tn = "XSEM";
+	    else if (i.i_rdev == S_INSHD)
+		tn = "XSD";
+	    else {
+		tn = "XNAM";
+		(void) snpf(Namech, Namechl,
+		    "unknown Xenix special file type: %x", i.i_rdev);
+	    }
+	    break;
 	case IFIFO:
 	    tn = "FIFO";
 	    break;
@@ -644,11 +666,8 @@ process_node(na)
 #endif	/* defined(IFLNK) */
 
 	default:
-	    if (type > 9999)
-		(void) snpf(Lf->type, sizeof(Lf->type), "*%03d", type % 1000);
-	    else
-		(void) snpf(Lf->type, sizeof(Lf->type), "%4d", type);
-	    (void) snpf(Namech, Namechl, "unknown inode type");
+	    (void) snpf(Lf->type, sizeof(Lf->type), "%04o",
+		((type >> 12) & 0xfff));
 	    tn = NULL;
 	}
 	if (tn)
@@ -658,6 +677,7 @@ process_node(na)
  */
 	switch (Ntype) {
 	case N_BLK:
+	case N_CHR:
 	case N_FIFO:
 	case N_HSFS:
 
@@ -665,6 +685,7 @@ process_node(na)
 	case N_NFS:
 #endif	/* defined(HAS_NFS) */
 
+	case N_NM:
 	case N_REGLR:
 	    if (Lf->dev_def) {
 
@@ -696,7 +717,7 @@ process_node(na)
  * Test for specified file.
  */
 	if (Sfile && is_file_named((char *)NULL,
-				   ((type == IFCHR) || (type == IFBLK)) ? 1
+		((type == IFCHR) || (type == IFBLK) || (type == IFNAM)) ? 1
 									: 0))
 	    Lf->sf |= SELNM;
 
