@@ -32,7 +32,6 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1995 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dsock.c,v 1.11 2000/08/01 13:13:46 abe Exp $";
 #endif
 
 
@@ -73,7 +72,7 @@ process_socket(i)
 	struct sockaddr_in si;
 	struct un_dev ud;
 #endif	/* OSRV<500 */
-	
+
 	(void) snpf(Lf->type, sizeof(Lf->type), "sock");
 /*
  * Read socket.
@@ -82,7 +81,7 @@ process_socket(i)
 	    (void) enter_nm("No kernel socket table");
 	    return;
 	}
-	spa = Socktab + (minor(i->i_rdev) * sizeof(struct socket *));
+	spa = Socktab + (GET_MIN_DEV(i->i_rdev) * sizeof(struct socket *));
 	if (kread(spa, (char *)&sa, sizeof(sa))) {
 	    (void) snpf(Namech, Namechl, "can't read socket pointer at %s",
 		print_kptr(spa, (char *)NULL, 0));
@@ -232,10 +231,34 @@ process_socket(i)
 		    (void) ent_inaddr(la, lp, fa, fp, AF_INET);
 	    }
 	/*
-	 * Save sizes.
+	 * Save options, sizes, states and values.
 	 */
+
+#if	defined(HASSOOPT)
+	    Lf->lts.ltm = (unsigned int)s.so_linger;
+	    Lf->lts.opt = (unsigned int)s.so_options;
+	    Lf->lts.qlen = (unsigned int)s.so_qlen;
+	    Lf->lts.qlim = (unsigned int)s.so_qlimit;
+	    Lf->lts.qlens = Lf->lts.qlims = (unsigned char)1;
+	    if (ts && t.t_timer[TCPT_KEEP]) {
+		Lf->lts.opt |= SO_KEEPALIVE;
+		Lf->lts.kai = (unsigned long)t.t_timer[TCPT_KEEP];
+	    }
+#endif	/* defined(HASSOOPT) */
+
+#if	defined(HASSOSTATE)
+	    Lf->lts.ss = s.so_state;
+#endif	/* defined(HASSOSTATE) */
+
+
 	    if (ts) {
 
+#if	defined(HASTCPOPT)
+		Lf->lts.topt = (unsigned int)t.t_flags;
+		Lf->lts.mss = (unsigned long)t.t_maxseg;
+		Lf->lts.msss = (unsigned char)1;
+#endif	/* defined(HASTCPOPT) */
+
 #if	defined(HASTCPTPIQ)
 		Lf->lts.rq = (unsigned long)t.t_iqsize;
 		Lf->lts.sq = (unsigned long)t.t_qsize;
@@ -339,7 +362,7 @@ udp_tm(tm)
  */
 	if (!Lbolt)
 	    return;
-	if (Hz < 0 
+	if (Hz < 0
 	||  kread((KA_T)Lbolt, (char *)&lbolt, sizeof(lbolt))
 	||  tm >= lbolt
 	||  (et = (time_t)((lbolt - tm) / Hz)) == 0)
@@ -391,7 +414,7 @@ udp_tm(tm)
 	if (!(cp = (char *)malloc(len))) {
 	    (void) fprintf(stderr, "%s: no space for %d character UDP time\n",
 		Pn, len);
-	    Exit(1);
+	    Error();
 	}
 	(void) snpf(cp, len, "%s", buf);
 	Lf->nma = cp;
