@@ -32,7 +32,6 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dfile.c,v 1.13 2000/12/04 14:35:13 abe Exp $";
 #endif
 
 
@@ -147,7 +146,7 @@ hashSfile()
 		(void) fprintf(stderr,
 		    "%s: can't allocate space for %d clone hash buckets\n",
 		    Pn, SFCDHASH);
-		Exit(1);
+		Error();
 	    }
 	}
 	if (!(HbyFdi = (struct hsfile *)calloc((MALLOC_S)SFDIHASH,
@@ -156,7 +155,7 @@ hashSfile()
 	    (void) fprintf(stderr,
 		"%s: can't allocate space for %d (dev,ino) hash buckets\n",
 		Pn, SFDIHASH);
-	    Exit(1);
+	    Error();
 	}
 	if (!(HbyFrd = (struct hsfile *)calloc((MALLOC_S)SFRDHASH,
 					       sizeof(struct hsfile))))
@@ -164,7 +163,7 @@ hashSfile()
 	    (void) fprintf(stderr,
 		"%s: can't allocate space for %d rdev hash buckets\n",
 		Pn, SFRDHASH);
-	    Exit(1);
+	    Error();
 	}
 	if (!(HbyFsd = (struct hsfile *)calloc((MALLOC_S)SFFSHASH,
 					       sizeof(struct hsfile))))
@@ -172,7 +171,7 @@ hashSfile()
 	    (void) fprintf(stderr,
 		"%s: can't allocate space for %d file sys hash buckets\n",
 		Pn, SFFSHASH);
-	    Exit(1);
+	    Error();
 	}
 	if (!(HbyNm = (struct hsfile *)calloc((MALLOC_S)SFNMHASH,
 					      sizeof(struct hsfile))))
@@ -180,7 +179,7 @@ hashSfile()
 	    (void) fprintf(stderr,
 		"%s: can't allocate space for %d name hash buckets\n",
 		Pn, SFNMHASH);
-	    Exit(1);
+	    Error();
 	}
 	hs++;
 /*
@@ -196,14 +195,14 @@ hashSfile()
 		    HbyNmCt++;
 		} else if (i == 1) {
 		    if (s->type) {
-			sh = &HbyFdi[SFHASHDEVINO(major(s->dev),
-						  minor(s->dev),
+			sh = &HbyFdi[SFHASHDEVINO(GET_MAJ_DEV(s->dev),
+						  GET_MIN_DEV(s->dev),
 						  s->i,
 						  SFDIHASH)];
 			HbyFdiCt++;
 		    } else {
-			sh = &HbyFsd[SFHASHDEVINO(major(s->dev),
-						  minor(s->dev),
+			sh = &HbyFsd[SFHASHDEVINO(GET_MAJ_DEV(s->dev),
+						  GET_MIN_DEV(s->dev),
 						  0,
 						  SFFSHASH)];
 			HbyFsdCt++;
@@ -212,19 +211,20 @@ hashSfile()
 		    if (s->type
 		    &&  ((s->mode == S_IFCHR) || (s->mode == S_IFBLK)))
 		    {
-			sh = &HbyFrd[SFHASHRDEVI(major(s->dev),
-						 minor(s->dev),
-						 major(s->rdev),
-						 minor(s->rdev),
+			sh = &HbyFrd[SFHASHRDEVI(GET_MAJ_DEV(s->dev),
+						 GET_MIN_DEV(s->dev),
+						 GET_MAJ_DEV(s->rdev),
+						 GET_MIN_DEV(s->rdev),
 						 s->i,
 						 SFRDHASH)];
 			HbyFrdCt++;
 		    } else
 			continue;
 		} else {
-		    if (!hvc || (major(s->rdev) != cmaj))
+		    if (!hvc || (GET_MAJ_DEV(s->rdev) != cmaj))
 			continue;
-		    sh = &HbyCd[SFHASHDEVINO(0, minor(s->rdev), 0, SFCDHASH)];
+		    sh = &HbyCd[SFHASHDEVINO(0, GET_MIN_DEV(s->rdev), 0,
+					     SFCDHASH)];
 		    HbyCdCt++;
 		}
 		if (!sh->s) {
@@ -238,7 +238,7 @@ hashSfile()
 			(void) fprintf(stderr,
 			    "%s: can't allocate hsfile bucket for: %s\n",
 			    Pn, s->aname);
-			Exit(1);
+			Error();
 		    }
 		    sn->s = s;
 		    sn->next = sh->next;
@@ -284,11 +284,14 @@ is_file_named(p, nt, vt, ps)
 	if (!f && HbyCdCt && nt == N_STREAM && Lf->dev_def && Lf->rdev_def
 	&&  (Lf->dev == DevDev))
 	{
-	    for (sh = &HbyCd[SFHASHDEVINO(0, major(Lf->rdev), 0, SFCDHASH)];
+	    for (sh = &HbyCd[SFHASHDEVINO(0, GET_MAJ_DEV(Lf->rdev), 0,
+					  SFCDHASH)];
 		 sh;
 		 sh = sh->next)
 	    {
-		if ((s = sh->s) && (major(Lf->rdev) == minor(s->rdev))) {
+		if ((s = sh->s) && (GET_MAJ_DEV(Lf->rdev)
+				==  GET_MIN_DEV(s->rdev)))
+		{
 		    f = 1;
 		    break;
 		}
@@ -300,15 +303,15 @@ is_file_named(p, nt, vt, ps)
 	if (!f && HbyFdiCt && Lf->dev_def
 	&&  (Lf->inp_ty == 1 || Lf->inp_ty == 3))
 	{
-	    for (sh = &HbyFdi[SFHASHDEVINO(major(Lf->dev),
-					   minor(Lf->dev),
+	    for (sh = &HbyFdi[SFHASHDEVINO(GET_MAJ_DEV(Lf->dev),
+					   GET_MIN_DEV(Lf->dev),
 					   Lf->inode,
 					   SFDIHASH)];
 		 sh;
 		 sh = sh->next)
 	    {
 		if ((s = sh->s) && (Lf->dev == s->dev)
-		&&  ((ino_t)Lf->inode == s->i)) {
+		&&  (Lf->inode == s->i)) {
 		    f = 1;
 		    break;
 		}
@@ -318,16 +321,14 @@ is_file_named(p, nt, vt, ps)
  * Check for a file system match.
  */
 	if (!f && HbyFsdCt && Lf->dev_def) {
-	    for (sh = &HbyFsd[SFHASHDEVINO(major(Lf->dev), minor(Lf->dev), 0,
-					   SFFSHASH)];
+	    for (sh = &HbyFsd[SFHASHDEVINO(GET_MAJ_DEV(Lf->dev),
+					   GET_MIN_DEV(Lf->dev), 0, SFFSHASH)];
 		 sh;
 		 sh = sh->next)
 	    {
 		if ((s = sh->s) && Lf->dev == s->dev) {
-		    if (!(vt == VCHR && s->mode != S_IFCHR)) {
-			f = 1;
-			break;
-		    }
+		    f = 1;
+		    break;
 		}
 	    }
 	}
@@ -340,8 +341,10 @@ is_file_named(p, nt, vt, ps)
 	&&  Lf->rdev_def
 	&& (Lf->inp_ty == 1 || Lf->inp_ty == 3))
 	{
-	    for (sh = &HbyFrd[SFHASHRDEVI(major(Lf->dev), minor(Lf->dev),
-					  major(Lf->rdev), minor(Lf->rdev),
+	    for (sh = &HbyFrd[SFHASHRDEVI(GET_MAJ_DEV(Lf->dev),
+					  GET_MIN_DEV(Lf->dev),
+					  GET_MAJ_DEV(Lf->rdev),
+					  GET_MIN_DEV(Lf->rdev),
 					  Lf->inode, SFRDHASH)];
 		 sh;
 		 sh = sh->next)
@@ -382,6 +385,7 @@ is_file_named(p, nt, vt, ps)
 }
 
 
+#if	defined(HASPRINTDEV)
 /*
  * print_dev() - print device
  */
@@ -410,6 +414,170 @@ print_dev(lf, dev)
 	    ), (int)(*dev & L_MAXMIN));
 	return(buf);
 }
+#endif	/* defined(HASPRINTDEV) */
+
+
+#if	defined(HAS_V_PATH)
+
+/*
+ * Local definitions
+ */
+
+#define	VPRDLEN	((MAXPATHLEN + 7)/8)	/* v_path read length increment */
+
+
+/*
+ * print_v_path() - print path name from vnode's v_path pointer
+ */
+
+extern int
+print_v_path(lf)
+	struct lfile *lf;		/* local file structure */
+{
+	char buf[MAXPATHLEN+1];
+	unsigned char del = 0;
+	unsigned char aperr = 0;
+
+# if	defined(HASMNTSTAT)
+	struct stat sb;
+# endif	/* defined(HASMNTSTAT) */
+
+# if	defined(HASVXFS) && defined(HASVXFSRNL)
+	if (lf->is_vxfs && (lf->inp_ty == 1) && lf->fsdir) {
+	    if (print_vxfs_rnl_path(lf))
+		return(1);
+	}
+# endif	/* defined(HASVXFS) && defined(HASVXFSRNL) */
+
+	(void) read_v_path((KA_T)lf->V_path, buf, (size_t)sizeof(buf));
+	if (buf[0]) {
+
+# if	defined(HASMNTSTAT)
+	    if (!lf->mnt_stat && lf->dev_def && (lf->inp_ty == 1)) {
+
+	    /*
+	     * No problem was detected in applying stat(2) to this mount point.
+	     * If the device and inode for the file are known, it is probably
+	     * safe and worthwhile to apply stat(2) to the v_path.
+	     */
+		if (!statsafely(buf, &sb)) {
+
+		/*
+		 * The stat(2) succeeded.  See if the device and inode match.
+		 * If they both don't match, ignore the v_path.
+		 */
+		    if ((lf->dev != sb.st_dev)
+		    ||  (lf->inode != (INODETYPE)sb.st_ino)
+		    ) {
+			return(0);
+		    }
+		} else {
+
+		/*
+		 * The stat(2) failed.
+		 *
+		 * If the error reply is ENOENT and the -X option hasn't been
+		 * specified, ignore the v_path.
+		 *
+		 * If the error reply is ENOENT, the -X option has been
+		 * specified and the file's link count is zero, report the
+		 * v_path with the "(deleted)" notation.
+		 *
+		 * If the error reply is EACCES or EPERM, report the v_path,
+		 * followed by "(?)", because lsof probably lacks permission
+		 * to apply stat(2) to v_path.
+		 */
+		    switch (errno) {
+		    case EACCES:
+		    case EPERM:
+			aperr = 1;
+			break;
+		    case ENOENT:
+
+# if	defined(HASXOPT)
+			if (Fxopt && lf->nlink_def && !lf->nlink) {
+			    del = 1;
+			    break;
+			}
+# endif	/* defined(HASXOPT) */
+
+			return(0);
+		    default:
+			return(0);
+		    }
+		}
+	    }
+# endif	/* defined(HASMNTSTAT) */
+
+	/*
+	 * Print the v_path.
+	 */
+	    safestrprt(buf, stdout, 0);
+	    if (del)
+		safestrprt(" (deleted)", stdout, 0);
+	    else if (aperr)
+		safestrprt(" (?)", stdout, 0);
+	    return(1);
+	}
+	return(0);
+}
+
+
+/*
+ * read_v_path() - read path name from vnode's v_path pointer
+ */
+
+extern void
+read_v_path(ka, rb, rbl)
+	KA_T ka;			/* kernel path address */
+	char *rb;			/* receiving buffer */
+	size_t rbl;			/* receiving buffer length */
+{
+	char *ba;
+	size_t rl, tl;
+
+	*rb = '\0';
+	if (!ka)
+	    return;
+	for (ba = rb, tl = 0;
+	     tl < (rbl - 1);
+	     ba += rl, ka += (KA_T)((char *)ka + rl), tl += rl
+	) {
+
+	/*
+	 * Read v_path VPRDLEN bytes at a time until the local buffer is full
+	 * or a NUL byte is reached.
+	 */
+	    if ((rl = rbl - 1 - tl) > VPRDLEN)
+		rl = VPRDLEN;
+	    else if (rl < 1) {
+		*(rb + rbl - 1) = '\0';
+		break;
+	    }
+	    if (!kread(ka, ba, rl)) {
+		*(ba + rl) = '\0';
+		if (strchr(ba, '\0') < (ba + rl))
+		    break;
+	    } else {
+
+	    /*
+	     * Can't read a full buffer load; try reducing the length one
+	     * byte at a time until it reaches zero.  Stop here, since it
+	     * has been established that no more bytes can be read.
+	     */
+		for (rl--; rl > 0; rl--) {
+		    if (!kread(ka, ba, rl)) {
+			*(ba + rl) = '\0';
+			break;
+		    }
+		}
+		if (rl <= 0)
+		    *ba = '\0';
+		break;
+	    }
+	}
+}
+#endif	/* defined(HAS_V_PATH) */
 
 
 /*
