@@ -32,7 +32,6 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1996 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dproc.c,v 1.13 2001/04/26 15:57:56 abe Exp $";
 #endif
 
 #include "lsof.h"
@@ -81,6 +80,7 @@ gather_proc_info()
 	struct execinfo ex;
 	static struct fd_entry *fe;
 	struct fd_entry *f;
+	KA_T fa;
 	int i, nf;
 	MALLOC_S len;
 	static int nfea = 0;
@@ -89,6 +89,11 @@ gather_proc_info()
 	struct pid pids;
 	short pss, sf;
 	uid_t uid;
+
+#if	UNIXWAREV>=70103
+	struct pollx plx;
+#endif	/* UNIXWAREV>=70103 */
+
 /*
  * Examine proc structures and their associated information.
  */
@@ -178,16 +183,25 @@ gather_proc_info()
 		    (void) fprintf(stderr,
 			"%s: PID %d; no space for %d file descriptors\n",
 			Pn, pid, nf);
-		    Exit(1);
+		    Error();
 		}
 		nfea = nf;
 	    }
 	    if (kread((KA_T)p->p_fdtab.fdt_entrytab, (char *)fe, len))
 		continue;
 	    for (f = fe, i = 0; i < nf; f++, i++) {
-		if (f->fd_file && (f->fd_status & FD_INUSE)) {
+		if ((fa = (KA_T)f->fd_file) && (f->fd_status & FD_INUSE)) {
+
+#if	UNIXWAREV>=70103
+		    if (f->fd_flag & FPOLLED) {
+			if (kread(fa, (char *)&plx, sizeof(plx))
+			||  !(fa = (KA_T)plx.px_fp))
+			    continue;
+		    }
+#endif	/* UNIXWAREV>=70103 */
+
 		    alloc_lfile(NULL, i);
-		    process_file((KA_T)f->fd_file);
+		    process_file(fa);
 		    if (Lf->sf) {
 
 #if	defined(HASFSTRUCT)
@@ -233,7 +247,7 @@ get_clonemaj()
 	if (!(cd = (struct cdevsw *)malloc(len))) {
 	    (void) fprintf(stderr, "%s: can't allocate %d bytes for cdevsw\n",
 		Pn);
-	    Exit(1);
+	    Error();
 	}
 /*
  * Read the cdevsw[] from kernel memory.
@@ -299,7 +313,7 @@ get_kernel_access()
  * See if the non-KMEM memory file is readable.
  */
 	if (Memory && !is_readable(Memory, 1))
-	    Exit(1);
+	    Error();
 #endif	/* defined(WILLDROPGID) */
 
 /*
@@ -308,7 +322,7 @@ get_kernel_access()
 	if ((Kd = open(Memory ? Memory : KMEM, O_RDONLY, 0)) < 0) {
 	    (void) fprintf(stderr, "%s: can't open %s: %s\n", Pn,
 		Memory ? Memory : KMEM, strerror(errno));
-	    Exit(1);
+	    Error();
 	}
 
 #if	defined(WILLDROPGID)
@@ -322,7 +336,7 @@ get_kernel_access()
  * See if the name list file is readable.
  */
 	if (Nmlst && !is_readable(Nmlst, 1))
-	    Exit(1);
+	    Error();
 #endif	/* defined(WILLDROPGID) */
 
 /*
@@ -332,18 +346,18 @@ get_kernel_access()
         if (nlist(Nmlst ? Nmlst : N_UNIX, Nl) < 0) {
 	    (void) fprintf(stderr, "%s: can't read kernel name list from %s\n",
 		Pn, Nmlst ? Nmlst : N_UNIX);
-	    Exit(1);
+	    Error();
 	}
 	if (get_Nl_value("var", Drive_Nl, &v) < 0 || !v
 	||  kread((KA_T)v, (char *)&Var, sizeof(Var))) {
 	    (void) fprintf(stderr,
 		"%s: can't read system configuration info\n", Pn);
-	    Exit(1);
+	    Error();
 	}
 	if (get_Nl_value("proc", Drive_Nl, &Pract) < 0 || !Pract) {
 	    (void) fprintf(stderr,
 		"%s: can't find active process chain pointer\n", Pn);
-	    Exit(1);
+	    Error();
 	}
 	if (get_Nl_value("sgdnops", Drive_Nl, &Sgdnops) < 0 || !Sgdnops)
 	    Sgdnops = (unsigned long)0;
@@ -497,27 +511,27 @@ readfsinfo()
 	if ((Fsinfomax = sysfs(GETNFSTYP)) == -1) {
 	    (void) fprintf(stderr, "%s: sysfs(GETNFSTYP) error: %s\n",
 		Pn, strerror(errno));
-	    Exit(1);
-	} 
+	    Error();
+	}
 	if (Fsinfomax == 0)
 	    return;
 	if (!(Fsinfo = (char **)malloc((MALLOC_S)(Fsinfomax * sizeof(char *)))))
 	{
 	    (void) fprintf(stderr, "%s: no space for sysfs info\n", Pn);
-	    Exit(1);
+	    Error();
 	}
 	for (i = 1; i <= Fsinfomax; i++) {
 	    if (sysfs(GETFSTYP, i, buf) == -1) {
 		(void) fprintf(stderr, "%s: sysfs(GETFSTYP) error: %s\n",
 		    Pn, strerror(errno));
-		Exit(1);
+		Error();
 	    }
 	    buf[FSTYPSZ] = '\0';
 	    len = strlen(buf) + 1;
 	    if (!(Fsinfo[i-1] = (char *)malloc((MALLOC_S)len))) {
 		(void) fprintf(stderr,
 		    "%s: no space for file system entry %s\n", Pn, buf);
-		Exit(1);
+		Error();
 	    }
 	    (void) snpf(Fsinfo[i-1], len, "%s", buf);
 	}
@@ -545,14 +559,14 @@ read_proc()
 	    if ((Npa = Var.v_proc) < 1) {
 		(void) fprintf(stderr, "%s: bad proc table size: %d\n",
 		    Pn, Var.v_proc);
-		Exit(1);
+		Error();
 	    }
 	    Npa += PROCINCR;
 	    len = (MALLOC_S)(Npa * sizeof(struct proc));
 	    if (!(P = (struct proc *)malloc(len))) {
 		(void) fprintf(stderr, "%s: no space for %d proc structures\n",
 		    Pn, Npa);
-		Exit(1);
+		Error();
 	    }
 	}
 /*
@@ -587,7 +601,7 @@ read_proc()
 			(void) fprintf(stderr,
 			    "%s: can't realloc %d proc table entries (%d)\n",
 			    Pn, Npa, len);
-			Exit(1);
+			Error();
 		    }
 		    p = &P[Np];
 		}
@@ -617,7 +631,7 @@ read_proc()
  */
 	if (try >= PROCTRYLM) {
 	    (void) fprintf(stderr, "%s: can't read proc table\n", Pn);
-	    Exit(1);
+	    Error();
 	}
 	if (Np < Npa && !RptTm) {
 
@@ -628,7 +642,7 @@ read_proc()
 	    if (!(P = (struct proc *)realloc((MALLOC_P *)P, len))) {
 		(void) fprintf(stderr,
 		    "%s: can't reduce proc table to %d entries\n", Pn, Np);
-		Exit(1);
+		Error();
 	    }
 	    Npa = Np;
 	}
