@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: lsof.h,v 1.65 2011/09/07 19:13:49 abe Exp $
+ * $Id: lsof.h,v 1.70 2018/03/26 21:50:45 abe Exp $
  */
 
 
@@ -80,6 +80,59 @@ struct l_dev {
 };
 
 
+# if	defined(HASEPTOPTS)
+/*
+ * End point definitions
+ */
+
+#define	CHEND_PIPE	0x01		/* pipe endpoint ID */
+#define	CHEND_PTY	0x02		/* pseudoterminal endpoint ID */
+
+#  if	defined(HASUXSOCKEPT)
+#define	CHEND_UXS	0x04		/* UNIX socket endpoint ID */
+#  endif	/* defined(HASUXSOCKEPT) */
+
+#define	CHEND_NETS	0x08		/* INET socket endpoint ID */
+#define	CHEND_PSXMQ	0x10		/* Posix MQ endpoint ID */
+
+# if	defined(HASIPv6)
+#define	CHEND_NETS6	0x20		/* INET6 socket endpoint ID */
+# endif	/* defined(HASIPv6) */
+
+#define CHEND_EVTFD	0x40		/* eventfd endpoint ID */
+
+#define	EPT_PIPE	0x01		/* process has pipe file */
+#define	EPT_PIPE_END	0x02		/* process has pipe end point file */
+#define	EPT_PTY		0x04		/* process has a pseudoterminal file */
+#define	EPT_PTY_END	0x08		/* process has a pseudoterminal end
+					 * point file */
+
+#  if	defined(HASUXSOCKEPT)
+#define	EPT_UXS		0x10		/* process has a UNIX socket file */
+#define	EPT_UXS_END	0x20		/* process has a UNIX socket end point
+					 * file */
+#  endif	/* defined(HASUXSOCKEPT) */
+
+#define	EPT_NETS	0x40		/* process has a INET socket file */
+#define	EPT_NETS_END	0x80		/* process has a INET socket end point
+					 * file */
+
+#define	EPT_PSXMQ	0x100		/* process has a POSIX MQ file*/
+#define	EPT_PSXMQ_END	0x200		/* process has a POSIX MQ end point
+					 * file*/
+
+# if	defined(HASIPv6)
+#define	EPT_NETS6	0x400		/* process has a INET6 socket file */
+#define	EPT_NETS6_END	0x800		/* process has a INET6 socket end point
+					 * file */
+# endif	/* defined(HASIPv6) */
+
+#define	EPT_EVTFD	0x1000		/* process has a eventfd file*/
+#define	EPT_EVTFD_END	0x2000		/* process has a eventfd end point
+					 * file*/
+# endif	/* defined(HASEPTOPTS) */
+
+
 /*
  * FILE_FLAG column names
  */
@@ -135,6 +188,7 @@ struct l_dev {
 #define	FF_NOTOSTOP	"NOTO"
 #define	FF_NSHARE	"NSH"
 #define	FF_OLRMIRROR	"OLRM"
+#define	FF_PATH		"PATH"
 #define	FF_POSIX_AIO	"PAIO"
 #define	FF_POSIX_PIPE	"PP"
 #define	FF_RAIOSIG	"RAIO"
@@ -158,6 +212,7 @@ struct l_dev {
 #define	FF_SYNCRON	"SWR"
 #define	FF_TCP_MDEVONLY	"TCPM"
 #define	FF_TERMIO	"TIO"
+#define	FF_TMPFILE	"TMPF"
 #define	FF_TRUNC	"TR"
 #define	FF_VHANGUP	"VH"
 #define	FF_VTEXT	"VTXT"
@@ -266,18 +321,21 @@ extern int optind;
 
 # if	!defined(LOGINML)
 #  if	defined(HASUTMPX)
-static struct utmpx dummy_utmp;		/* to get login name length */
-#define	LOGINML		sizeof(dummy_utmp.ut_user)
+#define	LOGINML		sizeof(((struct utmpx *)0)->ut_user)
 					/* login name length */
 #  else	/* !defined(HASUTMPX) */
-static struct utmp dummy_utmp;		/* to get login name length */
-#define	LOGINML		sizeof(dummy_utmp.ut_name)
+#define	LOGINML		sizeof(((struct utmp *)0)->ut_name)
 					/* login name length */
 #  endif	/* defined(HASUTMPX) */
 # endif	/* !defined(LOGINML) */
 
 #define	LPROCINCR	128		/* Lproc[] allocation increment */
-#define	LSOF_URL	"ftp://lsof.itap.purdue.edu/pub/tools/unix/lsof/"
+#define	LSOF_GITHUB_URL	"https://github.com/lsof-org"
+#define	LSOF_REPO	"lsof"
+#define	LSOF_BRANCH	"master"
+#define	LSOF_REPO_URL	LSOF_GITHUB_URL "/" LSOF_REPO
+#define	LSOF_FAQ_URL	LSOF_REPO_URL "/blob/" LSOF_BRANCH "/00FAQ"
+#define	LSOF_MAN_URL	LSOF_REPO_URL "/blob/" LSOF_BRANCH "/Lsof.8"
 #define	MIN_AF_ADDR	sizeof(struct in_addr)
 					/* minimum AF_* address length */
 
@@ -298,57 +356,59 @@ static struct utmp dummy_utmp;		/* to get login name length */
 #define	N_REGLR		0		/* regular file system node */
 #define	N_AFS		1		/* AFS node */
 #define	N_AFPFS		2		/* Apple Darwin AppleShare */
-#define	N_AUSX		3		/* Auspex LFS node */
-#define	N_AUTO		4		/* automount node */
-#define	N_BLK		5		/* block device node */
-#define	N_CACHE		6		/* cached file system node */
-#define	N_CDFS		7		/* CD-ROM node */
-#define	N_CFS		8		/* CFS node */
-#define	N_CHR		9		/* character device node */
-#define	N_COM		10		/* streams common device node */
-#define	N_CTFSADIR	11		/* Solaris CTFS adir node */
-#define	N_CTFSBUND	12		/* Solaris CTFS bundle node */
-#define	N_CTFSCDIR	13		/* Solaris CTFS cdir node */
-#define	N_CTFSCTL	14		/* Solaris CTFS ctl node */
-#define	N_CTFSEVT	15		/* Solaris CTFS events node */
-#define	N_CTFSLATE	16		/* Solaris CTFS latest node */
-#define	N_CTFSROOT	17		/* Solaris CTFS root node */
-#define	N_CTFSSTAT	18		/* Solaris CTFS status node */
-#define	N_CTFSSYM	19		/* Solaris CTFS symbolic node */
-#define	N_CTFSTDIR	20		/* Solaris CTFS type node */
-#define	N_CTFSTMPL	21		/* Solaris CTFS template node */
-#define	N_DEV		22		/* DEV FS node */
-#define	N_DOOR		23		/* DOOR node */
-#define	N_FD		24		/* FD node */
-#define	N_FIFO		25		/* FIFO node */
-#define	N_HSFS		26		/* High Sierra node */
-#define	N_KERN		27		/* BSD /kern node */
-#define	N_LOFS		28		/* loopback node */
-#define	N_MNT		29		/* mount file system device node */
-#define	N_MPC		30		/* multiplexed device node */
-#define	N_MVFS		31		/* multi-volume file system node (?) */
-#define	N_NFS		32		/* NFS node */
-#define	N_NFS4		33		/* NFS version 4 node */
-#define	N_NM		34		/* named file system node */
-#define	N_OBJF		35		/* objfs file system node */
-#define	N_PCFS		36		/* PC file system node */
-#define	N_PIPE		37		/* pipe device node */
-#define	N_PORT		38		/* port node */
-#define	N_PROC		39		/* /proc node */
-#define	N_PSEU		49		/* pseudofs node */
-#define	N_SAMFS		41		/* Solaris SAM-FS */
-#define	N_SANFS		42		/* AIX SANFS */
-#define	N_SDEV		43		/* Solaris sdev file system node */
-#define	N_SHARED	44		/* Solaris sharedfs */
-#define	N_SOCK		45		/* sock_vnodeops node */
-#define	N_SPEC		46		/* spec_vnodeops node */
-#define	N_STREAM	47		/* stream node */
-#define	N_TMP		48		/* tmpfs node */
-#define	N_UFS		49		/* UNIX file system node */
-#define	N_UNKN		50		/* unknown node type */
-#define	N_VXFS		51		/* Veritas file system node */
-#define	N_XFS		52		/* XFS node */
-#define	N_ZFS		53		/* ZFS node */
+#define	N_ANON_INODE	3		/* Linux anon_inodefs node */
+#define	N_AUSX		4		/* Auspex LFS node */
+#define	N_AUTO		5		/* automount node */
+#define	N_BLK		6		/* block device node */
+#define	N_CACHE		7		/* cached file system node */
+#define	N_CDFS		8		/* CD-ROM node */
+#define	N_CFS		9		/* CFS node */
+#define	N_CHR		10		/* character device node */
+#define	N_COM		11		/* streams common device node */
+#define	N_CTFSADIR	12		/* Solaris CTFS adir node */
+#define	N_CTFSBUND	13		/* Solaris CTFS bundle node */
+#define	N_CTFSCDIR	14		/* Solaris CTFS cdir node */
+#define	N_CTFSCTL	15		/* Solaris CTFS ctl node */
+#define	N_CTFSEVT	16		/* Solaris CTFS events node */
+#define	N_CTFSLATE	17		/* Solaris CTFS latest node */
+#define	N_CTFSROOT	18		/* Solaris CTFS root node */
+#define	N_CTFSSTAT	19		/* Solaris CTFS status node */
+#define	N_CTFSSYM	20		/* Solaris CTFS symbolic node */
+#define	N_CTFSTDIR	21		/* Solaris CTFS type node */
+#define	N_CTFSTMPL	22		/* Solaris CTFS template node */
+#define	N_DEV		23		/* DEV FS node */
+#define	N_DOOR		24		/* DOOR node */
+#define	N_FD		25		/* FD node */
+#define	N_FIFO		26		/* FIFO node */
+#define	N_HSFS		27		/* High Sierra node */
+#define	N_KERN		28		/* BSD /kern node */
+#define	N_LOFS		29		/* loopback node */
+#define	N_MNT		30		/* mount file system device node */
+#define	N_MPC		31		/* multiplexed device node */
+#define	N_MVFS		32		/* multi-volume file system node (?) */
+#define	N_NFS		33		/* NFS node */
+#define	N_NFS4		34		/* NFS version 4 node */
+#define	N_NM		35		/* named file system node */
+#define	N_OBJF		36		/* objfs file system node */
+#define	N_PCFS		37		/* PC file system node */
+#define	N_PIPE		38		/* pipe device node */
+#define	N_PORT		39		/* port node */
+#define	N_PROC		40		/* /proc node */
+#define	N_PSEU		41		/* pseudofs node */
+#define	N_SAMFS		42		/* Solaris SAM-FS */
+#define	N_SANFS		43		/* AIX SANFS */
+#define	N_SDEV		44		/* Solaris sdev file system node */
+#define	N_SHARED	45		/* Solaris sharedfs */
+#define	N_SOCK		46		/* sock_vnodeops node */
+#define	N_SPEC		47		/* spec_vnodeops node */
+#define	N_STREAM	48		/* stream node */
+#define	N_TMP		49		/* tmpfs node */
+#define	N_UFS		50		/* UNIX file system node */
+#define	N_UNKN		51		/* unknown node type */
+#define	N_VXFS		52		/* Veritas file system node */
+#define	N_XFS		53		/* XFS node */
+#define	N_ZFS		54		/* ZFS node */
+#define	N_MQUEUE	55		/* Posix mqueue node on Linux */
 
 # if	!defined(OFFDECDIG)
 #define	OFFDECDIG	8		/* maximum number of digits in the
@@ -363,6 +423,9 @@ static struct utmp dummy_utmp;		/* to get login name length */
 
 #define	RPTTM		15		/* default repeat seconds */
 #define	RTD		" rtd"		/* root directory fd name */
+#define	TASKCMDL	9		/* maximum number of characters from
+					 * command name to print in TASKCMD
+					 * column */
 #define TCPTPI_FLAGS	0x0001		/* report TCP/TPI socket options and
 					 * state, and TCP_NODELAY state */
 #define	TCPTPI_QUEUES	0x0002		/* report TCP/TPI queue lengths */
@@ -436,8 +499,10 @@ extern int PpidColW;
 #define SZTTL		"SIZE"
 #define	SZOFFTTL	"SIZE/OFF"
 extern int SzOffColW;
-#define	TIDTTL		"TID"
-extern	int TidColW;
+#define	TASKCMDTTL	"TASKCMD"
+extern	int TaskCmdColW;
+#define	TASKTIDTTL	"TID"
+extern	int TaskTidColW;
 #define TYPETTL		"TYPE"
 extern int TypeColW;
 #define	USERTTL		"USER"
@@ -469,12 +534,39 @@ extern int ZoneColW;
 #define	SELZONE		0x1000		/* select zone (-z) */
 #define	SELEXCLF	0x2000		/* file selection excluded */
 #define	SELTASK		0x4000		/* select tasks (-K) */
+#define	SELPINFO	0x8000		/* selected for pipe info (cleared in
+					 * link_lfile() */
+#define	SELUXSINFO	0x10000		/* selected for UNIX socket info;
+					 * cleared in link_lfile() */
+#define	SELPTYINFO	0x20000		/* selected for pseudoterminal info;
+					 * cleared in link_lfile() */
+#define	SELNETSINFO	0x40000		/* selected for INET socket info;
+					 * cleared in link_lfile() */
+#define SELPSXMQINFO	0x80000		/* selected for POSIX MQ socket info;
+					   cleared in link_lfile() */
+#define	SELNETS6INFO	0x100000	/* selected for INET6 socket info;
+					 * cleared in link_lfile() */
+#define	SELEVTFDINFO	0x200000	/* selected for evetnfd info;
+					 * cleared in link_lfile() */
+
 #define	SELALL		(SELCMD|SELCNTX|SELFD|SELNA|SELNET|SELNM|SELNFS|SELPID|SELUID|SELUNX|SELZONE|SELTASK)
 #define	SELPROC		(SELCMD|SELCNTX|SELPGID|SELPID|SELUID|SELZONE|SELTASK)
 					/* process selecters */
 #define	SELFILE		(SELFD|SELNFS|SELNLINK|SELNM)	/* file selecters */
 #define	SELNW		(SELNA|SELNET|SELUNX)		/* network selecters */
 
+
+/*
+ * Exit Status
+ */
+
+enum ExitStatus {
+	LSOF_SUCCESS,
+	LSOF_ERROR,
+};
+#define LSOF_SEARCH_FAILURE (FsearchErr? LSOF_ERROR: LSOF_SUCCESS)
+
+
 /*
  * Structure definitions
  */
@@ -490,6 +582,8 @@ struct afsnode {			/* AFS pseudo-node structure */
 };
 # endif	/* defined(HAS_AFS) */
 
+extern int AllProc;
+
 # if	defined(HAS_STD_CLONE)
 struct clone {
 	int dx;			/* index of device entry in Devtp[] */
@@ -532,7 +626,7 @@ struct int_lst {
 typedef struct lsof_rx {		/* regular expression table entry */
 	char *exp;			/* original regular expression */
 	regex_t cx;			/* compiled expression */
-	int mc;				/* match count */ 
+	int mc;				/* match count */
 } lsof_rx_t;
 extern lsof_rx_t *CmdRx;
 extern int NCmdRxU;
@@ -544,6 +638,7 @@ struct pff_tab {			/* print file flags table structure */
 };
 # endif	/* defined(HASFSTRUCT) */
 
+
 struct seluid {
 	uid_t uid;			/* User ID */
 	char *lnm;			/* specified login name (NULL = none) */
@@ -570,7 +665,7 @@ extern struct str_lst *Cmdl;
 extern int CmdLim;
 extern int Cmdni;
 extern int Cmdnx;
- 
+
 # if	defined(HASSELINUX)
 typedef struct cntxlist {
 	char *cntx;			/* zone name */
@@ -620,6 +715,8 @@ extern int Fnfs;
 extern int Fnlink;
 extern int Foffset;
 extern int Fovhd;
+extern int FeptE;
+
 extern int Fport;
 
 # if	!defined(HASNORPC_H)
@@ -628,6 +725,7 @@ extern int FportMap;
 
 extern int Fpgid;
 extern int Fppid;
+extern int FsearchErr;
 extern int Fsize;
 extern int Fsv;
 extern int FsvByf;
@@ -671,8 +769,10 @@ extern struct fieldsel FieldSel[];
 extern int Hdr;
 
 enum IDType {PGID, PID};
+extern int  IgnTasks;
 extern char *InodeFmt_d;
 extern char *InodeFmt_x;
+extern int LastPid;
 
 struct lfile {
 	char access;
@@ -701,6 +801,18 @@ struct lfile {
 
 	unsigned char nlink_def;	/* link count definition status */
 	unsigned char off_def;		/* offset definition status */
+
+# if	defined(HASEPTOPTS)
+	unsigned char chend;		/* communication channel endpoint
+					 * file */
+	int eventfd_id;			/* evntfd id taken from
+					   /proc/$pid/fdinfo */
+#  if	defined(HASPTYEPT)
+	int tty_index;			/* pseudoterminal index of slave side
+					 * (if this is the master side) */
+#  endif	/* defined(HASPTYEPT) */
+# endif	/* defined(HASEPTOPTS) */
+
 	unsigned char rdev_def;		/* rdev definition status */
 	unsigned char sz_def;		/* size definition status */
 
@@ -711,7 +823,7 @@ struct lfile {
 	char fd[FDLEN];
 	char iproto[IPROTOL];
 	char type[TYPEL];
-	short sf;			/* select flags -- SEL* symbols */
+	unsigned int sf;		/* select flags -- SEL* symbols */
 	int ch;				/* VMPC channel: -1 = none */
 	int ntype;			/* node type -- N_* value */
 	SZOFFTYPE off;
@@ -827,7 +939,7 @@ extern struct lfile *Lf, *Plf;
 
 struct lproc {
 	char *cmd;			/* command name */
- 
+
 # if	defined(HASSELINUX)
 	char *cntx;			/* security context */
 # endif	/* defined(HASSELINUX) */
@@ -836,10 +948,15 @@ struct lproc {
 	short pss;			/* state: 0 = not selected
 				 	 *	  1 = wholly selected
 				 	 *	  2 = partially selected */
+# if	defined(HASEPTOPTS)
+	short ept;			/* end point status -- EPT_* values */
+# endif	/* defined(HASEPTOPTS) */
+
 	int pid;			/* process ID */
 
 # if	defined(HASTASKS)
 	int tid;			/* task ID */
+	char *tcmd;			/* task command name */
 # endif	/* HASTASKS */
 
 	int pgid;			/* process group ID */
@@ -854,6 +971,7 @@ struct lproc {
 };
 extern struct lproc *Lp, *Lproc;
 
+extern int MaxFd;
 extern char *Memory;
 extern int MntSup;
 extern char *MntSupP;
@@ -935,9 +1053,11 @@ extern int Procsrch;
 
 extern int PrPass;
 extern int RptTm;
+extern int RptMaxCount;
 extern struct l_dev **Sdev;
-extern int Selall;
+extern int SelAll;
 extern int Selflags;
+extern int SelProc;
 extern int Setgid;
 extern int Selinet;
 extern int Setuidroot;
@@ -949,7 +1069,9 @@ extern char *SzOffFmt_0t;
 extern char *SzOffFmt_d;
 extern char *SzOffFmt_dv;
 extern char *SzOffFmt_x;
-extern int TaskPrtFl;
+extern int TaskCmdLim;
+extern int TaskPrtCmd;
+extern int TaskPrtTid;
 extern int TcpStAlloc;
 extern unsigned char *TcpStI;
 extern int TcpStIn;
