@@ -31,13 +31,6 @@
  * 4. This notice may not be removed or altered.
  */
 
-#ifndef lint
-static char copyright[] =
-"@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: main.c,v 1.55 2011/09/07 19:13:49 abe Exp $";
-#endif
-
-
 #include "lsof.h"
 
 
@@ -65,10 +58,12 @@ main(argc, argv)
 	int argc;
 	char *argv[];
 {
-	int ad, c, i, n, rv, se1, se2, ss;
+	enum ExitStatus rv;
+	int gopt_rv;
+	int ad, c, i, n, se1, se2, ss;
 	char *cp;
 	int err = 0;
-	int ev = 0;
+	enum ExitStatus ev = LSOF_SUCCESS;
 	int fh = 0;
 	char *fmtr = (char *)NULL;
 	long l;
@@ -84,10 +79,11 @@ main(argc, argv)
 	struct str_lst *str, *strt;
 	int version = 0;
 	int xover = 0;
+	int pr_count = 0;
 
 #if	defined(HAS_STRFTIME)
 	char *fmt = (char *)NULL;
-	size_t fmtl;
+	size_t fmtl = (size_t)0;
 #endif	/* defined(HAS_STRFTIME) */
 
 #if	defined(HASZONES)
@@ -112,19 +108,36 @@ main(argc, argv)
 	else
 	    Pn = argv[0];
 /*
- * Close all file descriptors above 2.
+ * Close enough file descriptors above 2 that library functions will have
+ * open descriptors.
  *
  * Make sure stderr, stdout, and stdin are open descriptors.  Open /dev/null
  * for ones that aren't.  Be terse.
  *
  * Make sure umask allows lsof to define its own file permissions.
  */
-	for (i = 3, n = GET_MAX_FD(); i < n; i++)
+
+	if ((MaxFd = (int) GET_MAX_FD()) < 53)
+	    MaxFd = 53;
+
+#if	defined(HAS_CLOSEFROM)
+	(void) closefrom(3);
+#else	/* !defined(HAS_CLOSEFROM) */
+#if	defined(SYS_close_range)
+	if (MaxFd > 3 && syscall(SYS_close_range, 3, MaxFd - 1, 0) == 0)
+	    goto closed;
+#endif
+	for (i = 3; i < MaxFd; i++)
 	    (void) close(i);
+#if	defined(SYS_close_range)
+  closed:
+#endif
+#endif	/* !defined(HAS_CLOSEFROM) */
+
 	while (((i = open("/dev/null", O_RDWR, 0)) >= 0) && (i < 2))
 	    ;
 	if (i < 0)
-	    Exit(1);
+	    Error();
 	if (i > 2)
 	    (void) close(i);
 	(void) umask(0);
@@ -147,14 +160,14 @@ main(argc, argv)
 	    Setuidroot = 1;
 	if (!(Namech = (char *)malloc(MAXPATHLEN + 1))) {
 	    (void) fprintf(stderr, "%s: no space for name buffer\n", Pn);
-	    Exit(1);
+	    Error();
 	}
 	Namechl = (size_t)(MAXPATHLEN + 1);
 /*
  * Create option mask.
  */
 	(void) snpf(options, sizeof(options),
-	    "?a%sbc:%sD:d:%sf:F:g:hi:%s%slL:%s%snNo:Op:Pr:%ss:S:tT:u:UvVwx:%s%s%s",
+	    "?a%sbc:%sD:d:%s%sf:F:g:hi:%s%slL:%s%snNo:Op:QPr:%ss:S:tT:u:UvVwx:%s%s%s",
 
 #if	defined(HAS_AFS) && defined(HASAOPT)
 	    "A:",
@@ -174,6 +187,12 @@ main(argc, argv)
 	    "",
 #endif	/* defined(HASEOPT) */
 
+#if	defined(HASEPTOPTS)
+	    "E",
+#else	/* !defined(HASEPTOPTS) */
+	    "",
+#endif	/* defined(HASEPTOPTS) */
+
 #if	defined(HASKOPT)
 	    "k:",
 #else	/* !defined(HASKOPT) */
@@ -181,7 +200,7 @@ main(argc, argv)
 #endif	/* defined(HASKOPT) */
 
 #if	defined(HASTASKS)
-	    "K",
+	    "K:",
 #else	/* !defined(HASTASKS) */
 	    "",
 #endif	/* defined(HASTASKS) */
@@ -219,7 +238,7 @@ main(argc, argv)
 #else	/* !defined(HASZONES) */
 	    "",
 #endif	/* defined(HASZONES) */
- 
+
 #if	defined(HASSELINUX)
 	    "Z:"
 #else	/* !defined(HASSELINUX) */
@@ -230,8 +249,8 @@ main(argc, argv)
 /*
  * Loop through options.
  */
-	while ((c = GetOpt(argc, argv, options, &rv)) != EOF) {
-	    if (rv) {
+	while ((c = GetOpt(argc, argv, options, &gopt_rv)) != EOF) {
+	    if (gopt_rv) {
 		err = 1;
 		continue;
 	    }
@@ -270,7 +289,7 @@ main(argc, argv)
 			    GOx2 = GObk[1];
 			}
 		    } else {
-			CmdLim = atoi(GOv);
+			CmdLim = TaskCmdLim = atoi(GOv);
 
 #if	defined(MAXSYSCMDL)
 			if (CmdLim > MAXSYSCMDL) {
@@ -311,15 +330,8 @@ main(argc, argv)
 	    case 'C':
 		Fncache = (GOp == '-') ? 0 : 1;
 		break;
+
 #endif	/* defined(HASNCACHE) */
-
-#if	defined(HASEOPT)
-	    case 'e':
-		if (enter_efsys(GOv, ((GOp == '+') ? 1 : 0)))
-		    err = 1;
-		break;
-#endif	/* defined(HASEOPT) */
-
 	    case 'd':
 		if (GOp == '+') {
 		    if (enter_dir(GOv, 0))
@@ -353,6 +365,20 @@ main(argc, argv)
 
 		}
 		break;
+
+#if	defined(HASEOPT)
+	    case 'e':
+		if (enter_efsys(GOv, ((GOp == '+') ? 1 : 0)))
+		    err = 1;
+		break;
+#endif	/* defined(HASEOPT) */
+
+#if	defined(HASEPTOPTS)
+	    case 'E':
+		FeptE = (GOp == '+') ? 2 : 1;
+		break;
+#endif	/* defined(HASEPTOPTS) */
+
 	    case 'f':
 		if (!GOv || *GOv == '-' || *GOv == '+') {
 		    Ffilesys = (GOp == '+') ? 2 : 1;
@@ -442,6 +468,11 @@ main(argc, argv)
 			    continue;
 #endif	/* !defined(HASPPID) */
 
+#if	!defined(HASTASKS)
+			if (FieldSel[i].id == LSOF_FID_TCMD)
+			    continue;
+#endif	/* !defined(HASTASKS) */
+
 #if	!defined(HASFSTRUCT)
 			if (FieldSel[i].id == LSOF_FID_CT
 			||  FieldSel[i].id == LSOF_FID_FA
@@ -449,7 +480,7 @@ main(argc, argv)
 			||  FieldSel[i].id == LSOF_FID_NI)
 			    continue;
 #endif	/* !defined(HASFSTRUCT) */
- 
+
 #if	defined(HASSELINUX)
 			if ((FieldSel[i].id == LSOF_FID_CNTX) && !CntxStatus)
 			    continue;
@@ -496,6 +527,11 @@ main(argc, argv)
 			    continue;
 #endif	/* !defined(HASPPID) */
 
+#if	!defined(HASTASKS)
+			if (FieldSel[i].id == LSOF_FID_TCMD)
+			    continue;
+#endif	/* !defined(HASTASKS) */
+
 #if	!defined(HASFSTRUCT)
 			if (FieldSel[i].id == LSOF_FID_CT
 			||  FieldSel[i].id == LSOF_FID_FA
@@ -521,6 +557,10 @@ main(argc, argv)
 
 			    if (i == LSOF_FIX_TERM)
 				Terminator = '\0';
+
+			    if (i == LSOF_FIX_OFFSET)
+				Foffset = 1;
+
 			    break;
 			}
 		    }
@@ -575,10 +615,27 @@ main(argc, argv)
 #endif	/* defined(HASKOPT) */
 
 #if	defined(HASTASKS)
-		case 'K':
+	    case 'K':
+		if (!GOv || *GOv == '-' || *GOv == '+') {
 		    Ftask = 1;
+		    IgnTasks = 0;
 		    Selflags |= SELTASK;
-		    break;
+		    if (GOv) {
+			GOx1 = GObk[0];
+			GOx2 = GObk[1];
+		    }
+		} else {
+		    if (!strcasecmp(GOv, "i")) {
+			Ftask = 0;
+			IgnTasks = 1;
+			Selflags &= ~SELTASK;
+		   } else {
+			(void) fprintf(stderr,
+			    "%s: -K not followed by i (but by %s)\n", Pn, GOv);
+			err = 1;
+		   }
+		}
+		break;
 #endif	/* defined(HASTASKS) */
 
 	    case 'l':
@@ -705,12 +762,17 @@ main(argc, argv)
 		if (enter_id(PID, GOv))
 		    err = 1;
 		break;
+	    case 'Q':
+		FsearchErr = 0;
+		break;
 	    case 'P':
 		Fport = (GOp == '-') ? 0 : 1;
 		break;
 	    case 'r':
-		if (GOp == '+')
-		    ev = rc = 1;
+		if (GOp == '+') {
+		    ev = LSOF_ERROR;
+		    rc = 1;
+		}
 		if (!GOv || *GOv == '-' || *GOv == '+') {
 		    RptTm = RPTTM;
 		    if (GOv) {
@@ -733,6 +795,16 @@ main(argc, argv)
 		     break;
 		while(*cp && (*cp == ' '))
 		    cp++;
+
+		if (*cp == 'c') {
+		    cp++;
+		    for (i = 0;
+			 *cp && isdigit((unsigned char)*cp);
+			 cp++)
+			i = (i * 10) + ((int)*cp - '0');
+		    RptMaxCount = i;
+		}
+
 		if (*cp != LSOF_FID_MARK) {
 		    GOx1 = GObk[0];
 		    GOx2 = GObk[1] + n;
@@ -756,7 +828,7 @@ main(argc, argv)
 			(void) fprintf(stderr,
 			    "%s: no space (%d) for <fmt> result: \"%s\"\n",
 			    Pn, (int)fmtl, cp);
-			    Exit(1);
+			    Error();
 		    }
 		    if (util_strftime(fmtr, fmtl - 1, fmt) < 1) {
 			(void) fprintf(stderr, "%s: illegal <fmt>: \"%s\"\n",
@@ -937,7 +1009,7 @@ main(argc, argv)
 		}
 		break;
 #endif	/* defined(HASZONES) */
- 
+
 #if	defined(HASSELINUX)
 	    case 'Z':
 		if (!CntxStatus) {
@@ -965,6 +1037,11 @@ main(argc, argv)
 		err = 1;
 	    }
 	}
+/*
+ * If IgnTasks is set, remove SELTASK from SelAll and SelProc.
+ */
+	SelAll = IgnTasks ? (SELALL & ~SELTASK) : SELALL;
+	SelProc = IgnTasks ? (SELPROC & ~SELTASK) : SELPROC;
 /*
  * Check for argument consistency.
  */
@@ -1098,7 +1175,7 @@ main(argc, argv)
 			 (MALLOC_S)(sizeof(struct seluid) * Nuid))))
 	    {
 		(void) fprintf(stderr, "%s: can't realloc UID table\n", Pn);
-		Exit(1);
+		Error();
 	    }
 	    Mxuid = Nuid;
 	}
@@ -1107,7 +1184,7 @@ main(argc, argv)
  */
 	if ((Cmdl && Cmdni) || CmdRx)
 	    Selflags |= SELCMD;
- 
+
 #if	defined(HASSELINUX)
 	if (CntxArg)
 	    Selflags |= SELCNTX;
@@ -1143,12 +1220,12 @@ main(argc, argv)
 		    "%s: no select options to AND via -a\n", Pn);
 		usage(1, 0, 0);
 	    }
-	    Selflags = SELALL;
+	    Selflags = SelAll;
 	} else {
 	    if (GOx1 >= argc && (Selflags & (SELNA|SELNET)) != 0
-	    &&  (Selflags & ~(SELNA|SELNET)) == 0)
+	    && (Selflags & ~(SELNA|SELNET)) == 0)
 		Selinet = 1;
-	    Selall = 0;
+	    AllProc = 0;
 	}
 /*
  * Get the device for DEVDEV_PATH.
@@ -1171,7 +1248,7 @@ main(argc, argv)
 		    (void) fprintf(stderr, "%s: can't stat(/dev): %s\n", Pn,
 		    strerror(se2));
 		}
-		Exit(1);
+		Error();
 	    }
 	}
 	DevDev = sb.st_dev;
@@ -1179,8 +1256,8 @@ main(argc, argv)
  * Process the file arguments.
  */
 	if (GOx1 < argc) {
-	    if (ck_file_arg(GOx1, argc, argv, Ffilesys, 0, (struct stat *)NULL))
-		usage(1, 0, 0);
+	    if (ck_file_arg(GOx1, argc, argv, Ffilesys, 0, (struct stat *)NULL, FsearchErr == 0))
+		Error();
 	}
 /*
  * Do dialect-specific initialization.
@@ -1228,7 +1305,7 @@ main(argc, argv)
  */
 	if (MntSup == 1) {
 	    (void) readmnt();
-	    Exit(0);
+	    Exit(LSOF_SUCCESS);
 	}
 #endif	/* defined(HASMNTSUP) */
 
@@ -1257,7 +1334,7 @@ main(argc, argv)
 		    if (!slp) {
 			(void) fprintf(stderr,
 			    "%s: no space for %d sort pointers\n", Pn, Nlproc);
-			Exit(1);
+			Error();
 		    }
 		}
 		for (i = 0; i < Nlproc; i++) {
@@ -1275,6 +1352,133 @@ main(argc, argv)
 		NcacheReload = 1;
 #endif	/* defined(HASNCACHE) */
 
+#if	defined(HASEPTOPTS)
+	    /*
+	     * If endpoint info has been requested, make sure it is coded for
+	     * printing.
+	     *
+	     * Lf contents must be preserved, since they may point to a
+	     * malloc()'d area, and since Lf is used throughout the printing
+	     * of the selected processes.
+	     */
+		if (FeptE) {
+		    lf = Lf;
+		/*
+		 * Scan all selected processes.
+		 */
+		    for (i = 0; i < Nlproc; i++) {
+			Lp = (Nlproc > 1) ? slp[i] : &Lproc[i];
+
+			/*
+			 * For processes that have been selected for printing
+			 * and have files that are the end point(s) of pipe(s),
+			 * process the file endpoints.
+			 */
+			    if (Lp->pss && (Lp->ept & EPT_PIPE))
+				(void) process_pinfo(0);
+			/*
+			 * Process POSIX MQ endpoints.
+			 */
+			    if (Lp->ept & EPT_PSXMQ)
+			      (void) process_psxmqinfo(0);
+
+# if	defined(HASUXSOCKEPT)
+			/*
+			 * For processes that have been selected for printing
+			 * and have files that are the end point(s) of UNIX
+			 * socket(s), process the file endpoints.
+			 */
+			    if (Lp->pss && (Lp->ept & EPT_UXS))
+				(void) process_uxsinfo(0);
+# endif	/* defined(HASUXSOCKEPT) */
+
+# if	defined(HASPTYEPT)
+			/*
+			 * For processes that have been selected for printing
+			 * and have files that are the end point(s) of pseudo-
+			 * terminal files(s), process the file endpoints.
+			 */
+			    if (Lp->pss && (Lp->ept & EPT_PTY))
+				(void) process_ptyinfo(0);
+# endif	/* defined(HASPTYEPT) */
+
+			/*
+			 * Process INET socket endpoints.
+			 */
+			    if (Lp->ept & EPT_NETS)
+				(void) process_netsinfo(0);
+
+# if	defined(HASIPv6)
+			/*
+			 * Process INET6 socket endpoints.
+			 */
+			    if (Lp->ept & EPT_NETS6)
+				(void) process_nets6info(0);
+# endif	/* defined(HASIPv6) */
+			/*
+			 * Process eventfd endpoints.
+			 */
+			    if (Lp->ept & EPT_EVTFD)
+				(void) process_evtfdinfo(0);
+		    }
+		/*
+		 * In a second pass, look for unselected endpoint files,
+		 * possibly selecting them for printing.
+		 */
+		    for (i = 0; i < Nlproc; i++) {
+			Lp = (Nlproc > 1) ? slp[i] : &Lproc[i];
+
+			/*
+			 * Process pipe endpoints.
+			 */
+			    if (Lp->ept & EPT_PIPE_END)
+				(void) process_pinfo(1);
+			/*
+			 * Process POSIX MQ endpoints.
+			 */
+			    if (Lp->ept & EPT_PSXMQ_END)
+			      (void) process_psxmqinfo(1);
+
+# if	defined(HASUXSOCKEPT)
+			/*
+			 * Process UNIX socket endpoints.
+			 */
+			    if (Lp->ept & EPT_UXS_END)
+				(void) process_uxsinfo(1);
+# endif	/* defined(HASUXSOCKEPT) */
+
+# if	defined(HASPTYEPT)
+			/*
+			 * Process pseudo-terminal endpoints.
+			 */
+			    if (Lp->ept & EPT_PTY_END)
+				(void) process_ptyinfo(1);
+# endif	/* defined(HASPTYEPT) */
+
+			/*
+			 * Process INET socket endpoints.
+			 */
+			    if (Lp->ept & EPT_NETS_END)
+				(void) process_netsinfo(1);
+
+#if	defined(HASIPv6)
+			/*
+			 * Process INET6 socket endpoints.
+			 */
+			    if (Lp->ept & EPT_NETS6_END)
+				(void) process_nets6info(1);
+# endif	/* defined(HASIPv6) */
+
+			/*
+			 * Process envetfd endpoints.
+			 */
+			    if (Lp->ept & EPT_EVTFD_END)
+				(void) process_evtfdinfo(1);
+		    }
+		    Lf = lf;
+		}
+#endif	/* defined(HASEPTOPTS) */
+
 	    /*
 	     * Print the selected processes and count them.
 	     *
@@ -1301,11 +1505,34 @@ main(argc, argv)
 	 * If conditional repeat mode is in effect, see if it's time to exit.
 	 */
 	    if (RptTm) {
+
+#if	defined(HASEPTOPTS)
+		(void) clear_pinfo();
+
+		(void) clear_psxmqinfo();
+
+# if	defined(HASUXSOCKEPT)
+		(void) clear_uxsinfo();
+# endif	/* defined(HASUXSOCKEPT) */
+
+# if	defined(HASPTYEPT)
+		(void) clear_ptyinfo();
+# endif	/* defined(HASPTYEPT) */
+
+		(void) clear_netsinfo();
+
+# if	defined(HASIPv6)
+		(void) clear_nets6info();
+# endif	/* defined(HASIPv6) */
+
+		(void) clear_evtfdinfo();
+#endif	/* defined(HASEPTOPTS) */
+
 		if (rc) {
 		    if (!n)
 			break;
 		    else
-			ev = 0;
+			ev = LSOF_SUCCESS;
 		}
 
 #if	defined(HAS_STRFTIME)
@@ -1347,13 +1574,15 @@ main(argc, argv)
 		Hdr = Nlproc = 0;
 		CkPasswd = 1;
 	    }
+	    if (RptMaxCount && (++pr_count == RptMaxCount))
+		RptTm = 0;
 	} while (RptTm);
 /*
  * See if all requested information was displayed.  Return zero if it
  * was; one, if not.  If -V was specified, report what was not displayed.
  */
 	(void) childx();
-	rv = 0;
+	rv = LSOF_SUCCESS;
 	for (str = Cmdl; str; str = str->next) {
 
 	/*
@@ -1361,20 +1590,20 @@ main(argc, argv)
 	 */
 	    if (str->f)
 		continue;
-	    rv = 1;
+	    rv = LSOF_SEARCH_FAILURE;
 	    if (Fverbose) {
 		(void) printf("%s: command not located: ", Pn);
 		safestrprt(str->str, stdout, 1);
 	    }
 	}
 	for (i = 0; i < NCmdRxU; i++) {
-	
+
 	/*
 	 * Check command regular expressions.
 	 */
 	    if (CmdRx[i].mc)
 		continue;
-	    rv = 1;
+	    rv = LSOF_SEARCH_FAILURE;
 	    if (Fverbose) {
 		(void) printf("%s: no command found for regex: ", Pn);
 		safestrprt(CmdRx[i].exp, stdout, 1);
@@ -1387,7 +1616,7 @@ main(argc, argv)
 	 */
 	    if (sfp->f)
 		continue;
-	    rv = 1;
+	    rv = LSOF_SEARCH_FAILURE;
 	    if (Fverbose) {
 		(void) printf("%s: no file%s use located: ", Pn,
 		    sfp->type ? "" : " system");
@@ -1400,7 +1629,7 @@ main(argc, argv)
 	 * Report on proc file system search results.
 	 */
 	    if (Procsrch && !Procfind) {
-		rv = 1;
+		rv = LSOF_SEARCH_FAILURE;
 		if (Fverbose) {
 		    (void) printf("%s: no file system use located: ", Pn);
 		    safestrprt(Mtprocfs ? Mtprocfs->dir : HASPROCFS, stdout, 1);
@@ -1411,7 +1640,7 @@ main(argc, argv)
 
 		for (pfi = Procfsid; pfi; pfi = pfi->next) {
 		    if (!pfi->f) {
-			rv = 1;
+			rv = LSOF_SEARCH_FAILURE;
 			if (Fverbose) {
 			    (void) printf("%s: no file use located: ", Pn);
 			    safestrprt(pfi->nm, stdout, 1);
@@ -1455,7 +1684,7 @@ main(argc, argv)
 	    }
 	    for (np = Nwad; np; np = np->next) {
 		if (!np->f && (cp = np->arg)) {
-		    rv = 1;
+		    rv = LSOF_SEARCH_FAILURE;
 		    if (Fverbose) {
 			(void) printf("%s: Internet address not located: ", Pn);
 			safestrprt(cp ? cp : "(unknown)", stdout, 1);
@@ -1468,7 +1697,7 @@ main(argc, argv)
 	/*
 	 * Report no Internet files located.
 	 */
-	    rv = 1;
+	    rv = LSOF_SEARCH_FAILURE;
 	    if (Fverbose)
 		(void) printf("%s: no Internet files located\n", Pn);
 	}
@@ -1481,7 +1710,7 @@ main(argc, argv)
 	 */
 	    for (i = 0; i < TcpNstates; i++) {
 		if (TcpStI[i] == 1) {
-		    rv = 1;
+		    rv = LSOF_SEARCH_FAILURE;
 		    if (Fverbose)
 			(void) printf("%s: TCP state not located: %s\n",
 			    Pn, TcpSt[i]);
@@ -1495,7 +1724,7 @@ main(argc, argv)
 	 */
 	    for (i = 0; i < UdpNstates; i++) {
 		if (UdpStI[i] == 1) {
-		    rv = 1;
+		    rv = LSOF_SEARCH_FAILURE;
 		    if (Fverbose)
 			(void) printf("%s: UDP state not located: %s\n",
 			    Pn, UdpSt[i]);
@@ -1509,7 +1738,7 @@ main(argc, argv)
 	/*
 	 * Report no NFS files located.
 	 */
-	    rv = 1;
+	    rv = LSOF_SEARCH_FAILURE;
 	    if (Fverbose)
 		(void) printf("%s: no NFS files located\n", Pn);
 	}
@@ -1520,7 +1749,7 @@ main(argc, argv)
 	 */
 	    if (Spid[i].f || Spid[i].x)
 		continue;
-	    rv = 1;
+	    rv = LSOF_SEARCH_FAILURE;
 	    if (Fverbose)
 		(void) printf("%s: process ID not located: %d\n",
 		    Pn, Spid[i].i);
@@ -1532,7 +1761,7 @@ main(argc, argv)
 	/*
 	 * Report no tasks located.
 	 */
-	    rv = 1;
+	    rv = LSOF_SEARCH_FAILURE;
 	    if (Fverbose)
 		(void) printf("%s: no tasks located\n", Pn);
 	}
@@ -1547,7 +1776,7 @@ main(argc, argv)
 	    for (i = 0; i < HASHZONE; i++) {
 		for (zp = ZoneArg[i]; zp; zp = zp->next) {
 		    if (!zp->f) {
-			rv = 1;
+			rv = LSOF_SEARCH_FAILURE;
 			if (Fverbose) {
 			    (void) printf("%s: zone not located: ", Pn);
 			    safestrprt(zp->zn, stdout, 1);
@@ -1557,7 +1786,7 @@ main(argc, argv)
 	    }
 	}
 #endif	/* defined(HASZONES) */
- 
+
 #if	defined(HASSELINUX)
 	if (CntxArg) {
 
@@ -1566,7 +1795,7 @@ main(argc, argv)
 	 */
 	    for (cntxp = CntxArg; cntxp; cntxp = cntxp->next) {
 		if (!cntxp->f) {
-		    rv = 1;
+		    rv = LSOF_SEARCH_FAILURE;
 		    if (Fverbose) {
 			(void) printf("%s: context not located: ", Pn);
 			safestrprt(cntxp->cntx, stdout, 1);
@@ -1583,7 +1812,7 @@ main(argc, argv)
 	 */
 	    if (Spgid[i].f || Spgid[i].x)
 		continue;
-	    rv = 1;
+	    rv = LSOF_SEARCH_FAILURE;
 	    if (Fverbose)
 		(void) printf("%s: process group ID not located: %d\n",
 		    Pn, Spgid[i].i);
@@ -1595,7 +1824,7 @@ main(argc, argv)
 	 */
 	    if (Suid[i].excl || Suid[i].f)
 		continue;
-	    rv = 1;
+	    rv = LSOF_SEARCH_FAILURE;
 	    if (Fverbose) {
 		if (Suid[i].lnm) {
 		    (void) printf("%s: login name (UID %lu) not located: ",
@@ -1609,7 +1838,7 @@ main(argc, argv)
 	if (!rv && rc)
 	    rv = ev;
 	if (!rv && ErrStat)
-	    rv = 1;
+	    rv = LSOF_ERROR;
 	Exit(rv);
 	return(rv);		/* to make code analyzers happy */
 }
@@ -1707,7 +1936,7 @@ GetOpt(ct, opt, rules, err)
 	 * option following the possible value, so that the next call
 	 * to GetOpt() will find it.
 	 */
-	    if(opt[GOx1][GOx2 + 1] != '\0') {
+	    if (opt[GOx1][GOx2 + 1] != '\0') {
 		GObk[0] = GOx1;
 		GObk[1] = ++GOx2;
 		GOv = &opt[GOx1++][GOx2];
@@ -1758,7 +1987,7 @@ sv_fmt_str(f)
 	if (!(cp = (char *)malloc(l))) {
 	    (void) fprintf(stderr,
 		"%s: can't allocate %d bytes for format: %s\n", Pn, (int)l, f);
-	    Exit(1);
+	    Error();
 	}
 	(void) snpf(cp, l, "%s", f);
 	return(cp);
