@@ -29,13 +29,6 @@
  * 4. This notice may not be removed or altered.
  */
 
-#ifndef lint
-static char copyright[] =
-"@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: print.c,v 1.54 2012/04/10 16:30:51 abe Exp $";
-#endif
-
-
 #include "lsof.h"
 
 
@@ -161,6 +154,7 @@ endnm(sz)
 static void
 fill_portmap()
 {
+	static int already_run = 0;
 	char buf[128], *cp, *nm;
 	CLIENT *c;
 	int h, port, pr;
@@ -176,6 +170,13 @@ fill_portmap()
 	int s = RPC_ANYSOCK;
 #endif	/* !defined(CAN_USE_CLNT_CREATE) */
 
+/*
+ * Make sure this is only run once.
+ */
+	if (already_run)
+		return;
+	already_run = 1;
+
 /*
  * Construct structures for communicating with the portmapper.
  */
@@ -212,7 +213,7 @@ fill_portmap()
  * and UDP members.
  */
 	for (; p; p = p->pml_next) {
-	
+
 	/*
 	 * Determine the port map entry's protocol; ignore all but TCP and UDP.
 	 */
@@ -255,7 +256,7 @@ fill_portmap()
 		(void) fprintf(stderr,
 		    "%s: can't allocate space for portmap entry: ", Pn);
 		safestrprt(cp, stderr, 1);
-		Exit(1);
+		Error();
 	    }
 	    if (!nl) {
 		(void) free((FREE_P *)nm);
@@ -269,7 +270,7 @@ fill_portmap()
 		(void) fprintf(stderr,
 		    "%s: can't allocate porttab entry for portmap: ", Pn);
 		safestrprt(nm, stderr, 1);
-		Exit(1);
+		Error();
 	    }
 	    pt->name = nm;
 	    pt->nl = nl;
@@ -333,7 +334,7 @@ fill_porttab()
 		(void) fprintf(stderr,
 		    "%s: can't allocate %d bytes for port %d name: %s\n",
 		    Pn, (int)(nl + 1), p, se->s_name);
-		Exit(1);
+		Error();
 	    }
 	    if (!nl) {
 		(void) free((FREE_P *)nm);
@@ -343,7 +344,7 @@ fill_porttab()
 		(void) fprintf(stderr,
 		    "%s: can't allocate porttab entry for port %d: %s\n",
 		    Pn, p, se->s_name);
-		Exit(1);
+		Error();
 	    }
 	    pt->name = nm;
 	    pt->nl = nl - 1;
@@ -435,7 +436,7 @@ gethostnm(ia, af)
 	if (!(np = mkstrcpy(hn, (MALLOC_S *)NULL))) {
 	    (void) fprintf(stderr, "%s: no space for host name: ", Pn);
 	    safestrprt(hn, stderr, 1);
-	    Exit(1);
+	    Error();
 	}
 /*
  * Add address/name entry to cache.  Allocate cache space in HCINC chunks.
@@ -449,7 +450,7 @@ gethostnm(ia, af)
 		hc = (struct hostcache *)realloc((MALLOC_P *)hc, len);
 	    if (!hc) {
 		(void) fprintf(stderr, "%s: no space for host cache\n", Pn);
-		Exit(1);
+		Error();
 	    }
 	}
 	hc[hcx].af = af;
@@ -476,7 +477,6 @@ lkup_port(p, pr, src)
 	MALLOC_S nl;
 	char *nm, *pn;
 	static char pb[128];
-	static int pm = 0;
 	struct porttab *pt;
 /*
  * If the hash buckets haven't been allocated, do so.
@@ -499,7 +499,7 @@ lkup_port(p, pr, src)
 		      (int)(2 * (PORTHASHBUCKETS * sizeof(struct porttab *))),
 		      (h & 1) ? "UDP" : "TCP",
 		      (h > 1) ? "portmap" : "port");
-		    Exit(1);
+		    Error();
 		}
 	    }
 	}
@@ -509,10 +509,8 @@ lkup_port(p, pr, src)
  * If we're looking up program names for portmapped ports, make sure the
  * portmap table has been loaded.
  */
-	if (FportMap && !pm) {
+	if (FportMap)
 	    (void) fill_portmap();
-	    pm++;
-	}
 #endif	/* !defined(HASNORPC_H) */
 
 /*
@@ -565,7 +563,7 @@ lkup_port(p, pr, src)
 	if (!(pt = (struct porttab *)malloc(sizeof(struct porttab)))) {
 	    (void) fprintf(stderr,
 		"%s: can't allocate porttab entry for port %d\n", Pn, p);
-	    Exit(1);
+	    Error();
 	}
 /*
  * Allocate space for the name; copy it to the porttab entry; and link the
@@ -577,7 +575,7 @@ lkup_port(p, pr, src)
 	    (void) fprintf(stderr,
 		"%s: can't allocate space for port name: ", Pn);
 	    safestrprt(pn, stderr, 1);
-	    Exit(1);
+	    Error();
 	}
 	pt->name = nm;
 	pt->nl = nl;
@@ -678,8 +676,10 @@ print_file()
 		PIDTTL);
 
 #if	defined(HASTASKS)
-	    if (TaskPrtFl)
-		(void) printf(" %*s", TidColW, TIDTTL);
+	    if (TaskPrtTid)
+		(void) printf(" %*s", TaskTidColW, TASKTIDTTL);
+	    if (TaskPrtCmd)
+		(void) printf(" %-*.*s", TaskCmdColW, TaskCmdColW, TASKCMDTTL);
 #endif	/* defined(HASTASKS) */
 
 #if	defined(HASZONES)
@@ -766,20 +766,35 @@ print_file()
 
 #if	defined(HASTASKS)
 /*
- * Size or print task ID.
+ * Size or print task ID and command name.
  */
 	if (!PrPass) {
+	    if ((cp = Lp->tcmd)) {
+		len = safestrlen(cp, 2);
+		if (TaskCmdLim && (len > TaskCmdLim))
+		    len = TaskCmdLim;
+		if (len > TaskCmdColW)
+		    TaskCmdColW = len;
+		TaskPrtCmd = 1;
+	    }
 	    if (Lp->tid) {
 		(void) snpf(buf, sizeof(buf), "%d", Lp->tid);
-		if ((len = strlen(buf)) > TidColW)
-		    TidColW = len;
-		TaskPrtFl = 1;
+		if ((len = strlen(buf)) >TaskTidColW)
+		    TaskTidColW = len;
+		TaskPrtTid = 1;
+	    }
+	} else {
+	    if (TaskPrtTid) {
+		if (Lp->tid)
+		    (void) printf(" %*d", TaskTidColW, Lp->tid);
+		else
+		    (void) printf(" %*s", TaskTidColW, "");
+	    }
+	    if (TaskPrtCmd) {
+		cp = Lp->tcmd ? Lp->tcmd : "";
+		printf(" ");
+		safestrprtn(cp, TaskCmdColW, stdout, 2);
 	    }
-	} else if (TaskPrtFl) {
-	    if (Lp->tid)
-		(void) printf(" %*d", TidColW, Lp->tid);
-	    else
-		(void) printf(" %*s", TidColW, "");
 	}
 #endif	/* defined(HASTASKS) */
 
@@ -893,7 +908,7 @@ print_file()
 			FsColW = len;
 		} else
 		    (void) printf(" %*.*s", FsColW, FsColW, cp);
-		    
+
 	    }
 # endif	/* !defined(HASNOFSADDR) */
 
@@ -1222,6 +1237,15 @@ addr_too_long:
 				src = 0;
 			}
 		    }
+#else
+		    /*
+		     * Just for suppressing warnings reported from compiler.
+		     *
+		     * src is referenced in lkup_port() only if "!defined(HASNORPC_H)"
+		     * is true. The condition here is !defined(HASNORPC_H) is false.
+		     * Therefore the value of src has no impact.
+		     */
+		    src = 1;
 #endif	/* !defined(HASNORPC_H) */
 
 		    if (strcasecmp(Lf->iproto, "TCP") == 0)
@@ -1273,7 +1297,16 @@ addr_too_long:
 void
 print_init()
 {
+
+/*
+ * Preset standard values.
+ */
 	PrPass = (Ffield || Fterse) ? 1 : 0;
+	LastPid = -1;
+	TaskPrtCmd = TaskPrtTid = 0;
+/*
+ * Size columns by their titles.
+ */
 	CmdColW = strlen(CMDTTL);
 	DevColW = strlen(DEVTTL);
 	FdColW = strlen(FDTTL);
@@ -1290,10 +1323,10 @@ print_init()
 	    SzOffColW = strlen(OFFTTL);
 	else
 	    SzOffColW = strlen(SZOFFTTL);
-	TaskPrtFl = 0;
 
 #if	defined(HASTASKS)
-	TidColW = strlen(TIDTTL);
+	TaskCmdColW = strlen(TASKCMDTTL);
+	TaskTidColW = strlen(TASKTIDTTL);
 #endif	/* defined(HASTASKS) */
 
 	TypeColW = strlen(TYPETTL);
@@ -1973,7 +2006,7 @@ printiproto(p)
 	}
 	if (s)
 	    (void) snpf(Lf->iproto, sizeof(Lf->iproto), "%.*s", IPROTOL-1, s);
-	else {	
+	else {
 	    if (m < 0) {
 		for (i = 0, m = 1; i < IPROTOL-2; i++)
 		    m *= 10;
@@ -2105,7 +2138,7 @@ printname(nl)
 		    NcacheReload = 0;
 		}
 		if ((cp = ncache_lookup(buf, sizeof(buf), &fp))) {
-		    char *cp1; 
+		    char *cp1;
 
 		    if (*cp == '\0')
 			goto print_nma;
@@ -2324,7 +2357,7 @@ printuid(uid, ty)
 		if (stat("/etc/passwd", &sb) != 0) {
 		    (void) fprintf(stderr, "%s: can't stat(/etc/passwd): %s\n",
 			Pn, strerror(errno));
-		    Exit(1);
+		    Error();
 		}
 	    }
 	/*
@@ -2337,7 +2370,7 @@ printuid(uid, ty)
 		    (void) fprintf(stderr,
 			"%s: no space for %d byte UID cache hash buckets\n",
 			Pn, (int)(UIDCACHEL * (sizeof(struct uidcache *))));
-		    Exit(1);
+		    Error();
 		}
 		if (CkPasswd) {
 		    sbs = sb;
@@ -2395,7 +2428,7 @@ printuid(uid, ty)
 		    (void) fprintf(stderr,
 			"%s: no space for UID cache entry for: %lu, %s)\n",
 			Pn, (unsigned long)uid, pw->pw_name);
-		    Exit(1);
+		    Error();
 		}
 		(void) strncpy(upn->nm, pw->pw_name, LOGINML);
 		upn->nm[LOGINML] = '\0';
@@ -2745,13 +2778,13 @@ printunkaf(fam, ty)
 #endif	/* defined(pseudo_AF_KEY) */
 
 #if	defined(AF_KEY)		/* Security Association DB socket */
-	case AF_KEY:			
+	case AF_KEY:
 	    s = "KEY";
 	    break;
 #endif	/* defined(AF_KEY) */
 
 #if	defined(AF_NCA)		/* NCA socket */
-	case AF_NCA:			
+	case AF_NCA:
 	    s = "NCA";
 	    break;
 #endif	/* defined(AF_NCA) */
@@ -2763,7 +2796,7 @@ printunkaf(fam, ty)
 #endif	/* defined(AF_POLICY) */
 
 #if	defined(AF_PPP)		/* PPP socket */
-	case AF_PPP:			
+	case AF_PPP:
 	    s = "PPP";
 	    break;
 #endif	/* defined(AF_PPP) */
@@ -2810,7 +2843,7 @@ update_portmap(pt, pn)
 	    (void) fprintf(stderr,
 		"%s: can't allocate %d bytes for portmap name: %s[%s]\n",
 		Pn, (int)(nl + 1), pn, pt->name);
-	    Exit(1);
+	    Error();
 	}
 	(void) snpf(cp, nl + 1, "%s[%s]", pn, pt->name);
 	(void) free((FREE_P *)pt->name);
