@@ -29,12 +29,8 @@
  * 4. This notice may not be removed or altered.
  */
 
-#ifndef lint
 static char copyright[] =
-"@(#) Copyright 1998 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: usage.c,v 1.24 2006/03/27 23:04:25 abe Exp $";
-#endif
-
+"Copyright 1998 Purdue Research Foundation. All rights reserved.";
 
 #include "lsof.h"
 #include "version.h"
@@ -45,6 +41,7 @@ static char *rcsid = "$Id: usage.c,v 1.24 2006/03/27 23:04:25 abe Exp $";
  */
 
 _PROTOTYPE(static char *isnullstr,(char *s));
+_PROTOTYPE(static int print_in_col,(int col, char *cp));
 _PROTOTYPE(static void report_HASDCACHE,(int type, char *ttl, char *det));
 _PROTOTYPE(static void report_HASKERNIDCK,(char *pfx, char *verb));
 _PROTOTYPE(static void report_SECURITY,(char *pfx, char *punct));
@@ -71,6 +68,33 @@ isnullstr(s)
 }
 
 
+/*
+ * print_in_col() -- print character string in help column
+ */
+
+static int
+print_in_col(col, cp)
+	int col;				/* column number */
+	char *cp;				/* what to print */
+{
+	if (cp && *cp) {
+	    switch (col) {
+	    case 1:
+		(void) fprintf(stderr, "  %-23.23s", cp);
+		break;
+	    case 2:
+		(void) fprintf(stderr, "  %-25.25s", cp);
+		break;
+	    default:
+		(void) fprintf(stderr, "  %s\n", cp);
+		col = 0;
+	    }
+	    col++;
+	}
+	return(col);
+}
+
+
 /*
  * report_HASDCACHE() -- report device cache file state
  */
@@ -302,21 +326,21 @@ report_WARNDEVACCESS(pfx, verb, punct)
  */
 
 void
-usage(xv, fh, version)
-	int xv;				/* exit value */
+usage(err, fh, version)
+	int err;			/* it is called as part of error handlng? */
 	int fh;				/* ``-F ?'' status */
 	int version;			/* ``-v'' status */
 {
 	char buf[MAXPATHLEN+1], *cp, *cp1, *cp2;
-	int  i;
+	int col, i;
 
-	if (Fhelp || xv) {
+	if (Fhelp || err) {
 	    (void) fprintf(stderr, "%s %s\n latest revision: %s\n",
-		Pn, LSOF_VERSION, LSOF_URL);
-	    (void) fprintf(stderr, " latest FAQ: %sFAQ\n", LSOF_URL);
-	    (void) fprintf(stderr, " latest man page: %slsof_man\n", LSOF_URL);
+		Pn, LSOF_VERSION, LSOF_REPO_URL);
+	    (void) fprintf(stderr, " latest FAQ: %s\n", LSOF_FAQ_URL);
+	    (void) fprintf(stderr, " latest (non-formatted) man page: %s\n", LSOF_MAN_URL);
 	    (void) fprintf(stderr,
-		" usage: [-?ab%shlnNoOP%sstUvV%s]",
+		" usage: [-?ab%sh%slnNoOP%s%stUvV%s]",
 
 #if	defined(HASNCACHE)
 		"C",
@@ -324,12 +348,24 @@ usage(xv, fh, version)
 		"",
 #endif	/* defined(HASNCACHE) */
 
+#if	defined(HASTASKS)
+		"K",
+#else	/* !defined(HASTASKS) */
+		"",
+#endif	/* defined(HASTASKS) */
+
 #if	defined(HASPPID)
 		"R",
 #else	/* !defined(HASPPID) */
 		"",
 #endif	/* defined(HASPPID) */
 
+#if	defined(HASTCPUDPSTATE)
+		"",
+#else	/* !defined(HASTCPUDPSTATE) */
+		"s",
+#endif	/* defined(HASTCPUDPSTATE) */
+
 #if	defined(HASXOPT)
 # if	defined(HASXOPT_ROOT)
 		(Myuid == 0) ? "X" : ""
@@ -346,18 +382,29 @@ usage(xv, fh, version)
 	    (void) fprintf(stderr, " [-A A]");
 #endif	/* defined(HAS_AFS) && defined(HASAOPT) */
 
-	    (void) fprintf(stderr, " [+|-c c] [+|-d s] [+%sD D]",
-
+	    (void) fprintf(stderr, " [+|-c c] [+|-d s] [+%sD D]%s",
 #if	defined(HASDCACHE)
-		"|-"
+		"|-",
 #else	/* !defined(HASDCACHE) */
-		""
+		"",
 #endif	/* defined(HASDCACHE) */
 
+#if	defined(HASEPTOPTS)
+		" [+|-E]"
+#else	/* !defined(HASEPTOPTS) */
+		""
+#endif	/* defined(HASEPTOPTS) */
+
 		);
 
 	    (void) fprintf(stderr,
-		" [+|-f%s%s%s%s%s%s]\n [-F [f]] [-g [s]] [-i [i]]",
+		" %s[+|-f%s%s%s%s%s%s]\n [-F [f]] [-g [s]] [-i [i]]",
+
+#if	defined(HASEOPT)
+		"[+|-e s] ",
+#else	/* !defined(HASEOPT) */
+		"",
+#endif	/* defined(HASEOPT) */
 
 #if	defined(HASFSTRUCT)
 		"[",
@@ -413,44 +460,57 @@ usage(xv, fh, version)
 		);
 #endif	/* defined(HASMOPT) || defined(HASMNTSUP) */
 
+#if	!defined(HASNORPC_H)
+	    (void) fprintf(stderr, " [+|-M]");
+#endif	/* !defined(HASNORPC_H) */
+
 	    (void) fprintf(stderr,
-	        " [+|-M] [-o [o]]\n [-p s] [+|-r [t]] [-S [t]] [-T [t]]");
+		" [-o [o]] [-p s]\n [+|-r [t]]%s [-S [t]] [-T [t]]",
+
+#if	defined(HASTCPUDPSTATE)
+		" [-s [p:s]]"
+#else	/* !defined(HASTCPUDPSTATE) */
+		""
+#endif	/* defined(HASTCPUDPSTATE) */
+
+		);
 	    (void) fprintf(stderr, " [-u s] [+|-w] [-x [fl]]");
 
 #if	defined(HASZONES)
 	    (void) fprintf(stderr, " [-z [z]]");
+#else	/* !defined(HASZONES) */
+# if	defined(HASSELINUX)
+	    if (CntxStatus)
+		(void) fprintf(stderr, " [-Z [Z]]");
+# endif	/* defined(HASSELINUX) */
 #endif	/* defined(HASZONES) */
 
 	    (void) fprintf(stderr, " [--] [names]\n");
 	}
-	if (xv && !Fhelp) {
+	if (err && !Fhelp) {
 	    (void) fprintf(stderr,
 		"Use the ``-h'' option to get more help information.\n");
 	    if (!fh)
-    		Exit(xv);
+		Exit(err? LSOF_ERROR: LSOF_SUCCESS);
 	}
 	if (Fhelp) {
 	    (void) fprintf(stderr,
 		"Defaults in parentheses; comma-separated set (s) items;");
 	    (void) fprintf(stderr, " dash-separated ranges.\n");
-	    (void) fprintf(stderr, "  %-23.23s", "-?|-h list help");
-	    (void) fprintf(stderr, "  %-25.25s", "-a AND selections (OR)");
-	    (void) fprintf(stderr, "  %s\n", "-b avoid kernel blocks");
-	    (void) fprintf(stderr, "  %-23.23s", "-c c  cmd c, /c/[bix]");
+	    col = print_in_col(1, "-?|-h list help");
+	    col = print_in_col(col, "-a AND selections (OR)");
+	    col = print_in_col(col, "-b avoid kernel blocks");
+	    col = print_in_col(col,  "-c c  cmd c ^c /c/[bix]");
 	    (void) snpf(buf, sizeof(buf), "+c w  COMMAND width (%d)", CMDL);
-	    (void) fprintf(stderr, "  %-25.25s", buf);
-
-	    (void) fprintf(stderr, "  %s\n", 
+	    col = print_in_col(col, buf);
 
 #if	defined(HASNCACHE)
-		"-C no kernel name cache");
-#else	/* !defined(HASNCACHE) */
-		" ");
+	   col = print_in_col(col, "-C no kernel name cache");
 #endif	/* defined(HASNCACHE) */
 
-	    (void) fprintf(stderr, "  %-23.23s", "+d s  dir s files");
-	    (void) fprintf(stderr, "  %-25.25s", "-d s  select by FD set");
-	    (void) fprintf(stderr, "  %s\n", "+D D  dir D tree *SLOW?*");
+	    col = print_in_col(col, "+d s  dir s files");
+	    col = print_in_col(col,  "-d s  select by FD set");
+	    col = print_in_col(col,  "+D D  dir D tree *SLOW?*");
 
 #if	defined(HASDCACHE)
 	    if (Setuidroot)
@@ -465,10 +525,15 @@ usage(xv, fh, version)
 		cp = "?|i|b|r|u[path]";
 	    (void) snpf(buf, sizeof(buf), "-D D  %s", cp);
 #else	/* !defined(HASDCACHE) */
-	    (void) snpf(buf, sizeof(buf), " ");
+	    buf[0] = '\0';
 #endif	/* defined(HASDCACHE) */
 
-	    (void) fprintf(stderr, "  %-23.23s", buf);
+	    col = print_in_col(col, buf);
+
+#if	defined(HASEOPT)
+	    col = print_in_col(col, "+|-e s  exempt s *RISKY*");
+#endif	/* defined(HASEOPT) */
+
 	    (void) snpf(buf, sizeof(buf), "-i select IPv%s files",
 
 #if	defined(HASIPv6)
@@ -478,38 +543,40 @@ usage(xv, fh, version)
 #endif	/* defined(HASIPv6) */
 
 			  );
-	    (void) fprintf(stderr, "  %-25.25s", buf);
-	    (void) fprintf(stderr, "  %s\n", "-l list UID numbers");
-	    (void) fprintf(stderr, "  %-23.23s", "-n no host names");
-	    (void) fprintf(stderr, "  %-25.25s", "-N select NFS files");
-	    (void) fprintf(stderr, "  %s\n", "-o list file offset");
-	    (void) fprintf(stderr, "  %-23.23s", "-O avoid overhead *RISKY*");
-	    (void) fprintf(stderr, "  %-25.25s", "-P no port names");
-	    (void) fprintf(stderr, "  %s\n",
+	    col = print_in_col(col, buf);
+
+#if	defined(HASTASKS)
+/* DEBUG	    col = print_in_col(col, "-K list tasKs (threads)");	*/
+	    col = print_in_col(col, "-K [i] list|(i)gn tasKs");
+#endif	/* defined(HASTASKS) */
+
+	    col = print_in_col(col, "-l list UID numbers");
+	    col = print_in_col(col, "-n no host names");
+	    col = print_in_col(col, "-N select NFS files");
+	    col = print_in_col(col, "-o list file offset");
+	    col = print_in_col(col, "-O no overhead *RISKY*");
+	    col = print_in_col(col, "-P no port names");
+	    col = print_in_col(col, "-Q allow failed search");
 
 #if	defined(HASPPID)
-	 	"-R list paRent PID"
-#else	/* !defined(HASPPID) */
-		""
+	     col = print_in_col(col, "-R list paRent PID");
 #endif	/* defined(HASPPID) */
 
-	    );
-	    (void) fprintf(stderr, "  %-23.23s", "-s list file size");
-	    (void) fprintf(stderr, "  %-25.25s", "-t terse listing");
-	    (void) fprintf(stderr, "  %s\n", "-T disable TCP/TPI info");
-	    (void) fprintf(stderr, "  %-23.23s", "-U select Unix socket");
-	    (void) fprintf(stderr, "  %-25.25s", "-v list version info");
-	    (void) fprintf(stderr, "  %s\n", "-V verbose search");
+	    col = print_in_col(col,  "-s list file size");
+	    col = print_in_col(col,  "-t terse listing");
+	    col = print_in_col(col,  "-T disable TCP/TPI info");
+	    col = print_in_col(col,  "-U select Unix socket");
+	    col = print_in_col(col,  "-v list version info");
+	    col = print_in_col(col,  "-V verbose search");
 	    (void) snpf(buf, sizeof(buf), "+|-w  Warnings (%s)",
 
 #if	defined(WARNINGSTATE)
-		"-"
+		"-");
 #else	/* !defined(WARNINGSTATE) */
-		"+"
+		"+");
 #endif	/* defined(WARNINGSTATE) */
 
-	    );
-	    (void) fprintf(stderr, "  %-23.23s", buf);
+	    col = print_in_col(col, buf);
 
 #if	defined(HASXOPT)
 # if	defined(HASXOPT_ROOT)
@@ -524,15 +591,27 @@ usage(xv, fh, version)
 	    buf[0] = '\0';
 #endif	/* defined(HASXOPT) */
 
-	    if (buf[0])
-		(void) fprintf(stderr, "  %-25.25s", buf);
+	    col = print_in_col(col, buf);
 
 #if	defined(HASZONES)
-	    (void) fprintf(stderr,
-		(buf[0]) ? "  %s\n" : "  %-25.25s", "-z z  zone [z]");
+	    col = print_in_col(col, "-z z  zone [z]");
 #endif	/* defined(HASZONES) */
 
-	    (void) fprintf(stderr, "  %s\n", "-- end option scan");
+#if	defined(HASSELINUX)
+	    col = print_in_col(col, "-Z Z  context [Z]");
+#endif	/* defined(HASSELINUX) */
+
+	    col = print_in_col(col, "-- end option scan");
+	    if (col != 1)
+		(void) fprintf(stderr, "\n");
+
+#if	defined(HASEPTOPTS)
+	    (void) fprintf(stderr, "  %-36.36s  %s\n",
+		"-E display endpoint info",
+		"+E display endpoint info and files"
+	    );
+#endif	/* defined(HASEPTOPTS) */
+
 	    (void) fprintf(stderr, "  %-36.36s",
 		"+f|-f  +filesystem or -file names");
 
@@ -637,15 +716,20 @@ usage(xv, fh, version)
 # endif	/* defined(HASMNTSUP) */
 #endif	/* defined(HASMOPT) || defined(HASMNTSUP) */
 
+#if	!defined(HASNORPC_H)
 	    (void) snpf(buf, sizeof(buf), "+|-M   portMap registration (%s)",
 
-#if	defined(HASPMAPENABLED)
+# if	defined(HASPMAPENABLED)
 		"+"
-#else	/* !defined(HASPMAPENABLED) */
+# else	/* !defined(HASPMAPENABLED) */
 		"-"
-#endif	/* defined(HASPMAPENABLED) */
+# endif	/* defined(HASPMAPENABLED) */
 
 	    );
+#else	/* defined(HASNORPC_H) */
+	    buf[0] = '\0';
+#endif	/* !defined(HASNORPC_H) */
+
 	    (void) fprintf(stderr, "  %-36.36s", buf);
 	    (void) snpf(buf, sizeof(buf), "-o o   o 0t offset digits (%d)",
 		OFFDECDIG);
@@ -657,37 +741,37 @@ usage(xv, fh, version)
 	    (void) snpf(buf, sizeof(buf),
 		"-T %s%ss%s TCP/TPI %s%sSt%s (s) info",
 
-#if     defined(HASSOOPT) || defined(HASSOSTATE) || defined(HASTCPOPT)
+#if	defined(HASSOOPT) || defined(HASSOSTATE) || defined(HASTCPOPT)
 		"f",
 #else	/* !defined(HASSOOPT) && !defined(HASSOSTATE) && !defined(HASTCPOPT)*/
 		"",
 #endif	/* defined(HASSOOPT) || defined(HASSOSTATE) || defined(HASTCPOPT)*/
 
-#if 	defined(HASTCPTPIQ)
+#if	defined(HASTCPTPIQ)
 		"q",
 #else	/* !defined(HASTCPTPIQ) */
 		" ",
 #endif	/* defined(HASTCPTPIQ) */
 
-#if 	defined(HASTCPTPIW)
+#if	defined(HASTCPTPIW)
 		"w",
 #else	/* !defined(HASTCPTPIW) */
 		"",
 #endif	/* defined(HASTCPTPIW) */
 
-#if     defined(HASSOOPT) || defined(HASSOSTATE) || defined(HASTCPOPT)
+#if	defined(HASSOOPT) || defined(HASSOSTATE) || defined(HASTCPOPT)
 		"Fl,",
 #else	/* !defined(HASSOOPT) && !defined(HASSOSTATE) && !defined(HASTCPOPT)*/
 		"",
 #endif	/* defined(HASSOOPT) || defined(HASSOSTATE) || defined(HASTCPOPT)*/
 
-#if 	defined(HASTCPTPIQ)
+#if	defined(HASTCPTPIQ)
 		"Q,",
 #else	/* !defined(HASTCPTPIQ) */
 		"",
 #endif	/* defined(HASTCPTPIQ) */
 
-#if 	defined(HASTCPTPIW)
+#if	defined(HASTCPTPIW)
 		",Win"
 #else	/* !defined(HASTCPTPIW) */
 		""
@@ -722,9 +806,33 @@ usage(xv, fh, version)
 #endif	/* defined(HASIPv6) */
 
 		);
+
 	    (void) fprintf(stderr,
-		"  +|-r [t] repeat every t seconds (%d);", RPTTM);
-	    (void) fprintf(stderr, " + until no files, - forever\n");
+		"  +|-r [%s] repeat every t seconds (%d); %s",
+
+#if	defined(HAS_STRFTIME)
+		"t[m<fmt>]",
+#else	/* !defined(has_STRFTIME) */
+		"t",
+#endif	/* defined(HAS_STRFTIME) */
+
+		RPTTM,
+		" + until no files, - forever.\n");
+
+#if	defined(HAS_STRFTIME)
+	    (void) fprintf(stderr,
+		"       An optional suffix to t is m<fmt>; m must separate %s",
+		"t from <fmt> and\n");
+	    (void) fprintf(stderr, "      <fmt> is an strftime(3) format %s",
+		"for the marker line.\n");
+#endif	/* defined(HAS_STRFTIME) */
+
+#if	defined(HASTCPUDPSTATE)
+	    (void) fprintf(stderr,
+		"  -s p:s  exclude(^)|select protocol (p = TCP|UDP) states");
+	    (void) fprintf(stderr, " by name(s).\n");
+#endif	/* defined(HASTCPUDPSTATE) */
+
 	    (void) fprintf(stderr,
 		"  -u s   exclude(^)|select login|UID set s\n");
 	    (void) fprintf(stderr,
@@ -735,6 +843,11 @@ usage(xv, fh, version)
 	    (void) report_WARNDEVACCESS(NULL, NULL, ";");
 	    (void) report_HASKERNIDCK(" k", NULL);
 	    (void) report_HASDCACHE(0, NULL, NULL);
+
+#if	defined(DIALECT_WARNING)
+	    (void) fprintf(stderr, "WARNING: %s\n", DIALECT_WARNING);
+#endif	/* defined(DIALECT_WARNING) */
+
 	}
 	if (fh) {
 	    (void) fprintf(stderr, "%s:\tID    field description\n", Pn);
@@ -778,6 +891,14 @@ usage(xv, fh, version)
 		    continue;
 #endif	/* !defined(HASZONES) */
 
+#if	defined(HASSELINUX)
+		if ((FieldSel[i].id == LSOF_FID_CNTX) && !CntxStatus)
+		    continue;
+#else	/* !defined(HASSELINUX) */
+		if (FieldSel[i].id == LSOF_FID_CNTX)
+		    continue;
+#endif	/* !defined(HASSELINUX) */
+
 		(void) fprintf(stderr, "\t %c    %s\n",
 		    FieldSel[i].id, FieldSel[i].nm);
 	    }
@@ -795,19 +916,18 @@ usage(xv, fh, version)
 	 */
 	    (void) fprintf(stderr, "%s version information:\n", Pn);
 	    (void) fprintf(stderr, "    revision: %s\n", LSOF_VERSION);
-	    (void) fprintf(stderr, "    latest revision: %s\n", LSOF_URL);
-	    (void) fprintf(stderr, "    latest FAQ: %sFAQ\n",
-		LSOF_URL);
-	    (void) fprintf(stderr, "    latest man page: %slsof_man\n",
-		LSOF_URL);
+	    (void) fprintf(stderr, "    copyright notice: %s\n", copyright);
+	    (void) fprintf(stderr, "    latest revision: %s\n", LSOF_REPO_URL);
+	    (void) fprintf(stderr, "    latest FAQ: %s\n",
+		LSOF_FAQ_URL);
+	    (void) fprintf(stderr, "    latest (non-formatted) man page: %s\n",
+		LSOF_MAN_URL);
 
 #if	defined(LSOF_CINFO)
 	    if ((cp = isnullstr(LSOF_CINFO)))
 		(void) fprintf(stderr, "    configuration info: %s\n", cp);
 #endif	/* defined(LSOF_CINFO) */
 
-	    if ((cp = isnullstr(LSOF_CCDATE)))
-		(void) fprintf(stderr, "    constructed: %s\n", cp);
 	    cp = isnullstr(LSOF_HOST);
 	    if (!(cp1 = isnullstr(LSOF_LOGNAME)))
 		cp1 = isnullstr(LSOF_USER);
@@ -844,7 +964,12 @@ usage(xv, fh, version)
 	    (void) report_SECURITY("    ", ".\n");
 	    (void) report_WARNDEVACCESS("    ", "are", ".\n");
 	    (void) report_HASKERNIDCK("    K", "is");
+
+#if	defined(DIALECT_WARNING)
+	    (void) fprintf(stderr, "    WARNING: %s\n", DIALECT_WARNING);
+#endif	/* defined(DIALECT_WARNING) */
+
 	    (void) report_HASDCACHE(1, "    ", "\t");
 	}
-	Exit(xv);
+	Exit(err? LSOF_ERROR: LSOF_SUCCESS);
 }
